/* 
 *	Copyright (c) 1997-98 
 *	NorthEast Parallel Architectures Center, Syracuse University. 
 * All Rights Reserved. Permission to use, copy, modify, and distribute this
 * software and its documentation for educational, research and non-profit
 * purposes, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph appear in all
 * copies. Permission to incorporate this software into commercial products may
 * be obtained by contacting the NorthEast Parallel Architectures Center. 
 *
 * The software program and documentation are supplied "as is",
 * without any accompanying services from NPAC. NPAC does not
 * warrant that the operation of the program will be uninterrupted or
 * error-free. The end-user understands that the program was developed for
 * research purposes and is advised not to rely exclusively on the program for
 * any reason.
 *
 */

package edu.syr.npac.webflow.impl.connection;

import java.util.*;

/**
 * Compute Queue Implementation
 * 
 * @version     0.8 28 September 1998
 * @author      Shrideep Pallickara
 *
 */

public class ComputeQueue  {
  private int length=0;
  private QueueWidget head;
  private QueueWidget tail;
  
  public ComputeQueue() {
    
  }
  

  /**
   *<a name="addItem()"></a>
   * Adds an Item to the message Queue
   *@param obj   The message to be added to the Message Queue, this could be
   *             either a Module type or any Object Type
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueue
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#removeItem()
   */
  
  public synchronized void addItem(Object object)  {
    if (object == null)
      throw new NullPointerException();
    
    tail = new QueueWidget(object, null, tail);
    
    if (head == null)
      head = tail;
    else
      tail.next.prev = tail;
    
    length++;
  }

  /**
   *<a name="addItemToTop()"></a>
   * Adds an Item to the top of the message Queue
   *@param obj   The message to be added to the Message Queue, this could be
   *             either a Module type or any Object Type
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueue
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#addItem()
   */
  
  public synchronized void addItemToTop(Object object)  {
    if (object == null)
      throw new NullPointerException();
    QueueWidget queueWidget = new QueueWidget(object, head, null);

    head = queueWidget;
    
    length++;
  }
  
  /**
   *<a name="nextItem()"></a>
   * Get the next Item from the message Queue
   *
   * @return object The next messageWidget in the computeQueue.
   *
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueue
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#removeAllItems()
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#addItem()
   */

  public synchronized Object nextItem() {
    if (head == null)
      return null;

    Object object = head.object;
    head = head.prev;

    if (head == null)
      tail = null;
    else
      head.next = null;
    length--;
    System.gc();
    return object;
  }

  /**
   *<a name="insertItem()"></a>
   *Inserts an item between 2 other items
   */
   public synchronized Object insertItem(Object object1, Object object3) {

     for (QueueWidget queueWidget = head; 
       queueWidget != null; queueWidget = queueWidget.prev) {
       if (queueWidget.object.equals(object3)) {
	 QueueWidget queueWidget2=
	   new QueueWidget(object1, queueWidget, queueWidget.next);
	 queueWidget2.prev.next = queueWidget2;
	 queueWidget2.next.prev = queueWidget2;
	 length++;
	 return object1;
       }
     }
     return null;
   }

  /**
   *<a name="removeItem()"></a>
   * Removes an Item from the compute Queue. This corresponds to the Object 
   * used as the parameter.
   *
   *@param obj  The message to be removed from the compute Queue, this could be
   *             either a Module type or any Object Type
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueue
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#removeAllItems()
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#addItem()
   */
  
  public synchronized Object removeItem(Object object1) {

    for (QueueWidget queueWidget = head; 
	 queueWidget != null; queueWidget = queueWidget.prev) {
      if (queueWidget.object.equals(object1)) {
	Object object2 = queueWidget.object;
	
	if (queueWidget.next == null)
	  head = queueWidget.prev;
	else
	  queueWidget.next.prev = queueWidget.prev;
	
	if (queueWidget.prev == null)
	  tail = queueWidget.next;
	else
	  queueWidget.prev.next = queueWidget.next;
	length--;
	return object2;
      }
    }
    return null;
  }
  
  /**
   *<a name="removeAllItems()"></a>
   * Removes all Widgets from the compute Queue. 
   * This corresponds to the Object  used as the parameter.
   *
   *@param obj  The message to be removed from the compute Queue, this could be
   *             either a Module type or any Object Type
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueue
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#nextItem()
   *@see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#addItem()
   */

  public synchronized void removeAllItems() {
    head = tail = null;
    length = 0;
  }

  /**
   * <a name="peekItem()"></a>
   *
   * Returns the head of the computeQueue
   * @see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#nextItem()
   * @see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#addItem()
   */
  
  public synchronized Object peekItem() {
    if (head == null)
      return null;
    else
      return head.object;
  }
  
  /**
   *<a name="dataAvailable()"></a>
   * Report the availability of data on the compute Queue
   *
   * @return true If there is data available
   * @return false If there is no data available
   *
   */

  public boolean dataAvailable() {
    if (head != null)
      return true;
    else
      return false;
  }
  
  /**
   * <a name="queueSize()"></a>
   * Query the queue Size, corresponds to the number of messages unprocessed
   * in the computeQueue.
   * @return size the compute Queue length
   * @see  edu.syr.npac.webflow.impl.connection.ComputeQueueQueue#dataAvailable()
   */

  public int queueSize() {
    return length;
  }
  
  /**
   *<a name="elements()"></a>
   * Return an enumeration of all the Widgets in the compute Queue
   * @return enum The QueueEnumerator
   *
   */

  public synchronized Enumeration elements()  {
    return new QueueEnumerator(head);
  }
  
  /**
   * <a name="toString()"></a>
   * Convert the entire computeQueue into a String
   * @return string The String Representation of all the Widgets in the Queue
   *
   */

  public String toString()  {
    StringBuffer stringBuffer = new StringBuffer();
    stringBuffer.append("Queue[");
    Enumeration enumeration = elements();

    for (int i = 0; i < length; i++) {
      stringBuffer.append(enumeration.nextElement().toString());
      if (i < length - 1)
	stringBuffer.append(", ");
    }
    stringBuffer.append("]");
    return stringBuffer.toString();
  }

  /**
   * <a name="toObjectArray()"></a>
   * Convert the entire computeQueue into an Object Array
   * @return Object[] The Object Representation of all the Widgets in the Queue
   *
   */

  public synchronized Object[] toObjectArray() {
    Object objectsArray[] = new Object[length];
    QueueWidget queueWidget = head;
    for (int i = 0; i < length; i++) {
	objectsArray[i] = queueWidget.object;
	queueWidget = queueWidget.next;
      }
    return objectsArray;
  }

}

class QueueWidget {
  Object object;
  QueueWidget prev;
  QueueWidget next;
  
  QueueWidget(Object object, 
	       QueueWidget queueWidget1, QueueWidget queueWidget2) {
    this.object = object;
    prev = queueWidget1;
    next = queueWidget2;
  }
}

class QueueEnumerator implements Enumeration {
  private QueueWidget head;

  QueueEnumerator(QueueWidget queueWidget) {
    head = queueWidget;
  }
  
  public boolean hasMoreElements() {
    if (head == null)
      return false;
    else
      return true;
  }
  
  public Object nextElement() {
    if (head == null)
      return null;
    Object object = head.object;
    head = head.prev;
    return object;
  }
}

