import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;

public class ClientSecuredComm {

    private PublicKey              serverPublicKey;
    private PrivateKey             clientPrivateKey;
    private SecretKey              secretKey;
    private SignedControl          sc;
    private CipherControl          cc;
    private MacControl             mc;
    private ClientHandShaking      chs;
    private ObjectInputStream      ois;
    private ObjectOutputStream     oos;
    private Socket                 socket;

    public ClientSecuredComm(Socket s) {
        socket = s;
    }

    public boolean init() {
        try {

            ois = new ObjectInputStream(socket.getInputStream());
            oos = new ObjectOutputStream(socket.getOutputStream());
            oos.flush();

            chs = new ClientHandShaking(ois, oos);
            if (!chs.init()) return false;

            clientPrivateKey = chs.getClientPrivateKey();
            serverPublicKey = chs.getServerPublicKey();
            secretKey = chs.getSecretKey();
          
            sc = new SignedControl();
            cc = new CipherControl();
            mc = new MacControl();
            mc.init(secretKey);

            return true;
        } catch (Exception ex) {
            System.err.println("ClientSecuredComm Exception-> " + ex);
            return false;
        }
    }

    public void setupMessage(String p, String u, long l) {
        // set the MacMessage to ReturnMessage
        UserInfo ui = new UserInfo(p, u, l);
        MacMessage mm = new MacMessage();
        mc.setMacMessage(mm);
        mc.setMessageType((int) 0);
        mc.setMessage(ui);
        mc.calc();

        // encipher MacMessage to get SealedObject
        cc.init(secretKey, 0);
        cc.setMacMessage(mc.getMacMessage());
        SealedObject so = cc.getSealedObject();

        // sign the Object
        sc.setSealedObject(so);
        sc.initSign(clientPrivateKey);
    }

    public void setupMessage(int i, float f) {
        // set the MacMessage to ReturnMessage
        TransactionMessage tm = new TransactionMessage(i, f);
        MacMessage mm = new MacMessage();
        mc.setMacMessage(mm);
        mc.setMessageType((int) 1);
        mc.setMessage(tm);
        mc.calc();

        // encipher MacMessage to get SealedObject
        cc.init(secretKey, 0);
        cc.setMacMessage(mc.getMacMessage());
        SealedObject so = cc.getSealedObject();

        // sign the Object
        sc.setSealedObject(so);
        sc.initSign(clientPrivateKey);
    }

    public void setupMessage(int i, String s) {
        // set the MacMessage to ReturnMessage
        ReturnMessage rm = new ReturnMessage(i, s);
        MacMessage mm = new MacMessage();
        mc.setMacMessage(mm);
        mc.setMessageType((int) 2);
        mc.setMessage(rm);
        mc.calc();

        // encipher MacMessage to get SealedObject
        cc.init(secretKey, 0);
        cc.setMacMessage(mc.getMacMessage());
        SealedObject so = cc.getSealedObject();

        // sign the Object
        sc.setSealedObject(so);
        sc.initSign(clientPrivateKey);
    }

    public void send() {
        try {
            oos.writeObject(sc.getSignedObject());
            oos.flush();
        } catch (Exception ex) {
            System.err.println("ClientSecureComm.send() Exception-> " + ex);
        }
    }

    public boolean receive() {
        try {
            SignedObject sio = (SignedObject)ois.readObject();
            sc.setSignedObject(sio);
            if (!sc.verifySign(serverPublicKey)) {
                mc.setMessage(null);
                mc.setMessageType(-1);
                return false;
            }
            SealedObject seo = sc.getSealedObject();
            cc.init(secretKey, 1);
            cc.setSealedObject(seo);
            MacMessage mm = cc.getMacMessage();
            mc.setMacMessage(mm);
            if (!mc.verify()) {
                mc.setMessage(null);
                mc.setMessageType(-1);
                return false;
            }
            if (mm.type != 2) {
                mc.setMessage(null);
                mc.setMessageType(-1);
                return false;
            }
            return true;
        } catch (Exception ex) {
            System.err.println("ClientSecureComm.receive() Exception-> " + ex);
            return false;
        }
    }

    public int getMessageType() {
        return this.mc.getMessageType();
    }

    public Object getMessage() {
        return this.mc.getMessage();
    }

    public void done() {
        try {
            socket.close();
            ois.close();
            oos.close();
        } catch (Exception ex) {
            System.err.println("ClientSecuredComm Exception-> " + ex);
        }
    }
}
