import java.io.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;

public class CoordinatorHandShaking {

    private ObjectInputStream   ois;
    private ObjectOutputStream  oos;
    private PublicKey           coordinatorPublicKey;
    private PublicKey           clientPublicKey;
    private PrivateKey          coordinatorPrivateKey;
    private SecretKey           secretKey;
    private CoordinatorKeyPair  ckp;

    public CoordinatorHandShaking(ObjectInputStream in, ObjectOutputStream out,
                                  CoordinatorKeyPair ck) {
        ois = in;
        oos = out;
        ckp = ck;
    }

    public boolean init() {
        try {
            // Initialize the DH protocol and exchange the secret key for cipher
            CoordinatorDHKeyAgreement dhKey = new CoordinatorDHKeyAgreement(ois, oos);
            secretKey = dhKey.agree();

            coordinatorPublicKey = ckp.getPublicKey();
            coordinatorPrivateKey = ckp.getPrivateKey();
            byte[] coordinatorEncodedPublicKey = ckp.getEncodedPublicKey();

            // System.err.println("Coordinator: " + toHexString(coordinatorEncodedPublicKey));

            // send Coordinator's public key to Client
            oos.writeObject(coordinatorEncodedPublicKey);
            oos.flush();
        
            // obtain Client's public key
            // This will be replaced by reading from client's certificate

            byte[] clientEncodedPublicKey = (byte[])ois.readObject();

            // System.err.println("Client's Encoded PublicKey: " + toHexString(clientEncodedPublicKey));

            KeyFactory coordinatorKeyFac = KeyFactory.getInstance("DSA");
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(clientEncodedPublicKey);
            clientPublicKey = coordinatorKeyFac.generatePublic(x509KeySpec);

            return true;
        } catch (Exception ex) {
            System.err.println("CoordinatorHandShaking.init() Exception-> " + ex);
            return false;
        }
    }

    public PublicKey getCoordinatorPublicKey() {
        return this.coordinatorPublicKey;
    }

    public PublicKey getClientPublicKey() {
        return this.clientPublicKey;
    }

    public PrivateKey getCoordinatorPrivateKey() {
        return this.coordinatorPrivateKey;
    }

    public SecretKey getSecretKey() {
        return this.secretKey;
    }

    public void byte2hex(byte b, StringBuffer buf) {
        char hexChars[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

        int low = (b & 0x0f);
        int high = (b & 0xf0) >> 4;

        buf.append(hexChars[high]);
        buf.append(hexChars[low]);
    }

    /*
     * Converts a byte array to hex string
     */
    public String toHexString(byte[] block) {
        StringBuffer buf = new StringBuffer();

        int len = block.length;

        for (int i = 0; i < len; i++) {
             byte2hex(block[i], buf);
             if (i < len-1) {
                 buf.append(":");
             }
        }
        return buf.toString();
    }
}
