import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;

public class ServerSecuredComm {
    private Socket                 socket;
    private PublicKey              clientPublicKey;
    private PrivateKey             serverPrivateKey;
    private SecretKey              secretKey;
    private SignedControl          sc;
    private CipherControl          cc;
    private MacControl             mc;
    private ServerHandShaking      shs;
    private boolean                action;
    private ObjectInputStream      ois;
    private ObjectOutputStream     oos;
    private ServerKeyPair          skp;

    public ServerSecuredComm(Socket s, ServerKeyPair sk) {
        socket = s;
        skp = sk;
    }

    public boolean init() {
        try {
            oos = new ObjectOutputStream(socket.getOutputStream());
            oos.flush();
            ois = new ObjectInputStream(socket.getInputStream());

            shs = new ServerHandShaking(ois, oos, skp);
            if (!shs.init()) return false;

            clientPublicKey = shs.getClientPublicKey();
            serverPrivateKey = shs.getServerPrivateKey();
            secretKey = shs.getSecretKey();

            sc = new SignedControl();
            cc = new CipherControl();
            mc = new MacControl();
            mc.init(secretKey);
            action = false;

            return true;
        } catch (Exception ex) {
            System.err.println("ServerSecuredComm Exception-> " + ex);
            return false;
        }
    }

    public boolean init_() {
        try {
            ois = new ObjectInputStream(socket.getInputStream());
            oos = new ObjectOutputStream(socket.getOutputStream());
            oos.flush();

            shs = new ServerHandShaking(ois, oos, skp);
            if (!shs.init_()) return false;

            clientPublicKey = shs.getClientPublicKey();
            serverPrivateKey = shs.getServerPrivateKey();
            secretKey = shs.getSecretKey();

            sc = new SignedControl();
            cc = new CipherControl();
            mc = new MacControl();
            mc.init(secretKey);
            action = false;

            return true;
        } catch (Exception ex) {
            System.err.println("ServerSecuredComm Exception-> " + ex);
            return false;
        }
    }

    public void setupMessage(int i, String s) {
        // set the MacMessage to ReturnMessage
        ReturnMessage rm = new ReturnMessage(i, s);
        MacMessage mm = new MacMessage();
        mc.setMacMessage(mm);
        mc.setMessageType((int) 2);
        mc.setMessage(rm);
        mc.calc();

        // encipher MacMessage to get SealedObject
        cc.init(secretKey, 0);
        cc.setMacMessage(mc.getMacMessage());
        SealedObject so = cc.getSealedObject();

        // sign the Object
        sc.setSealedObject(so);
        sc.initSign(serverPrivateKey);
    }

    public void send() throws Exception {
        oos.writeObject(sc.getSignedObject());
        oos.flush();
    }

    public boolean receive() throws Exception {
        try {
            SignedObject sio = (SignedObject)ois.readObject();
            sc.setSignedObject(sio);
            if (!sc.verifySign(clientPublicKey)) {
                mc.setMessage(null);
                mc.setMessageType(-1);
                return false;
            }

            SealedObject seo = sc.getSealedObject();
            cc.init(secretKey, 1);
            cc.setSealedObject(seo);
            MacMessage mm = cc.getMacMessage();
            mc.setMacMessage(mm);
            if (!mc.verify()) {
                mc.setMessage(null);
                mc.setMessageType(-1);
                return false;
            }
            return true;
        } catch (InterruptedIOException iioe) {
            System.err.println("ServerSecuredComm.receive() Exception-> " + iioe);
            return false;
        }
    }

    public int getMessageType() {
        return this.mc.getMessageType();
    }

    public Object getMessage() {
        return this.mc.getMessage();
    }

    public void done() {
        try { 
            socket.close();
            ois.close();
            oos.close();
        } catch (Exception ex) {
            System.err.println("ServerSecuredComm Exception-> " + ex);
        }
    }
}
