//  HEPsimulator
//  High Energy Particle Physics Simulator
//
//  Written by Mark Hale (mj.hale@ic.ac.uk)

import java.awt.*;
import JSci.physics.relativity.*;
import JSci.physics.quantum.*;
import JSci.physics.particles.*;

/**
* HEPsimulator.
* This is the main application class.
*
* @version 1.1
* @author Mark Hale (mj.hale@ic.ac.uk)
*/
public final class HEPsimulator extends Frame implements Runnable {
        private MenuBar mBar;
        private Menu mExit,mExp,mRes;
        private MenuItem mDev,mSet;
        private DeviceDialog dlgDevice;
        private SetupDialog dlgSetup;
        private ResultsDialog dlgResults;
        /**
        * The Particle Database.
        */
        private QuantumParticle particles[]=new QuantumParticle[38];

        private QuantumParticle ingoing[]=new QuantumParticle[2];
        private QuantumParticle outgoing[]=new QuantumParticle[2];
        private CMframe uni;
        /**
        * Constructor.
        */
        public HEPsimulator() {
                super("HEP simulator");
                resize(400,300);
                mBar=new MenuBar();
                mBar.add(mExit=new Menu("Exit"));
                mBar.add(mExp=new Menu("Experiment"));
                mExp.add(mDev=new MenuItem("Device"));
                mExp.add(mSet=new MenuItem("Setup"));
                mBar.add(mRes=new Menu("Results"));
                setMenuBar(mBar);
                loadDatabase();
                dlgDevice=new DeviceDialog(this,dlgSetup=new SetupDialog(this,particles));
        }
        public static void main(String args[]) {
                HEPsimulator sim=new HEPsimulator();
                sim.start();
        }
        public void start() {
                show();
        }
        public boolean handleEvent(Event evt) {
                if(evt.id==Event.WINDOW_DESTROY || evt.target.equals(mExit)) {
                        hide();
                        dlgDevice.dispose();
                        dlgSetup.dispose();
                        dispose();
                        System.exit(0);
                        return true;
                } else {
                        return super.handleEvent(evt);
                }
        }
        public boolean action(Event evt,Object obj) {
                if(evt.target.equals(mDev)) {
                        dlgDevice.show();
                } else if(evt.target.equals(mSet)) {
                        dlgSetup.show();
                } else if(evt.target.equals(mRes)) {
                        (new Thread(this)).start();
                }
                return true;
        }
        /**
        * Performs the experiment.
        */
        public void run() {
                uni=new CMframe();
                ingoing[0]=dlgSetup.getParticleA();
                ingoing[1]=dlgSetup.getParticleB();
                uni.setRelCMvel(ingoing[0],ingoing[1]);
                uni.addParticle(ingoing[0]);
                if(ingoing[1]!=null)
                        uni.addParticle(ingoing[1]);

                try {
                        for(int i=0;i<particles.length;i++) {
                                outgoing[0]=(QuantumParticle) particles[i].getClass().newInstance();
                                for(int j=i;j<particles.length;j++) {
                                        outgoing[1]=(QuantumParticle) particles[j].getClass().newInstance();
                                        if(uni.conserve(outgoing[0],outgoing[1])) {
                                                dlgResults=new ResultsDialog(this,ingoing,outgoing,uni.interact());
                                                dlgResults.show();
                                        }
                                }
                        }
                } catch(Exception e) {}
        }
        /**
        * Loads up the particle database.
        */
        private void loadDatabase() {
// LEPTIONS (12)
                particles[0]=new Electron();
                particles[1]=new Positron();
                particles[2]=new ElectronNeutrino();
                particles[3]=new AntiElectronNeutrino();
                particles[4]=new Muon();
                particles[5]=new AntiMuon();
                particles[6]=new MuonNeutrino();
                particles[7]=new AntiMuonNeutrino();
                particles[8]=new Tau();
                particles[9]=new AntiTau();
                particles[10]=new TauNeutrino();
                particles[11]=new AntiTauNeutrino();
// MESONS (8)
                particles[12]=new PiZero();
                particles[13]=new PiPlus();
                particles[14]=new PiMinus();
                particles[15]=new KPlus();
                particles[16]=new KMinus();
                particles[17]=new KZero();
                particles[18]=new AntiKZero();
                particles[19]=new Eta();
// BARYONS (18)
                particles[20]=new Proton();
                particles[21]=new AntiProton();
                particles[22]=new Neutron();
                particles[23]=new AntiNeutron();
                particles[24]=new Lambda();
                particles[25]=new AntiLambda();
                particles[26]=new SigmaPlus();
                particles[27]=new AntiSigmaPlus();
                particles[28]=new SigmaZero();
                particles[29]=new AntiSigmaZero();
                particles[30]=new SigmaMinus();
                particles[31]=new AntiSigmaMinus();
                particles[32]=new XiZero();
                particles[33]=new AntiXiZero();
                particles[34]=new XiMinus();
                particles[35]=new XiPlus();
                particles[36]=new Omega();
                particles[37]=new AntiOmega();
        }
}


/**
* DeviceDialog.
* Sets the experimental apparatus.
*
* @version 1.0
* @author Mark Hale
*/
class DeviceDialog extends Dialog {
        private SetupDialog dlgRel;
        private CheckboxGroup cbg;
        private Checkbox cbChamber;
        private Checkbox cbAccelerator;
        public DeviceDialog(Frame f,SetupDialog sd) {
                super(f,"Device Selection",true);
                resize(150,110);
                dlgRel=sd;
                setLayout(new GridLayout(3,1));
                cbg=new CheckboxGroup();
                add(cbChamber=new Checkbox("Bubble chamber",cbg,true));
                add(cbAccelerator=new Checkbox("Particle accelerator",cbg,false));
                add(new Button("OK"));
                dlgRel.particleB.disable();
                dlgRel.energyB.disable();
        }
        public boolean handleEvent(Event evt) {
                if(evt.id==Event.WINDOW_DESTROY || evt.target instanceof Button) {
                        hide();
                        return true;
                } else {
                        return super.handleEvent(evt);
                }
        }
        public boolean action(Event evt, Object obj) {
                if(evt.target.equals(cbChamber)) {
                        dlgRel.particleB.disable();
                        dlgRel.energyB.disable();
                } else if(evt.target.equals(cbAccelerator)) {
                        dlgRel.particleB.enable();
                        dlgRel.energyB.enable();
                }
                return true;
        }
}


/**
* SetupDialog.
* Sets the experimental parameters.
*
* @version 1.0
* @author Mark Hale
*/
class SetupDialog extends Dialog {
        public Choice particleA,particleB;
        public TextField energyA,energyB;
        private QuantumParticle array[];
        public SetupDialog(Frame f,QuantumParticle qpArray[]) {
                super(f,"Device Settings",true);
                resize(350,130);
                GridBagLayout gb=new GridBagLayout();
                GridBagConstraints c=new GridBagConstraints();
                Button ok=new Button("OK");
                array=qpArray;
                particleA=new Choice();
                particleB=new Choice();
                for(int n=0;n<array.length;n++) {
                        particleA.addItem(array[n].toString());
                        particleB.addItem(array[n].toString());
                }
                energyA=new TextField(Double.toString(array[0].restMass()));
                energyB=new TextField(Double.toString(array[0].restMass()));
                setLayout(gb);
                c.anchor=GridBagConstraints.CENTER;
                c.gridwidth=GridBagConstraints.RELATIVE;
                gb.setConstraints(particleA,c);
                add(particleA);
                c.gridwidth=GridBagConstraints.REMAINDER;
                gb.setConstraints(particleB,c);
                add(particleB);
                c.gridwidth=GridBagConstraints.RELATIVE;
                gb.setConstraints(energyA,c);
                add(energyA);
                c.gridwidth=GridBagConstraints.REMAINDER;
                gb.setConstraints(energyB,c);
                add(energyB);
                c.gridwidth=GridBagConstraints.REMAINDER;
                gb.setConstraints(ok,c);
                add(ok);
        }
        public boolean handleEvent(Event evt) {
                if(evt.id==Event.WINDOW_DESTROY || evt.target instanceof Button) {
                        hide();
                        return true;
                } else {
                        return super.handleEvent(evt);
                }
        }
        public boolean action(Event evt,Object obj) {
                if(evt.target.equals(particleA)) {
                        energyA.setText(Double.toString(array[particleA.getSelectedIndex()].restMass()));
                } else if(evt.target.equals(particleB)) {
                        energyB.setText(Double.toString(array[particleB.getSelectedIndex()].restMass()));
                } else if(evt.target.equals(energyA)) {
                        if(Double.valueOf(energyA.getText()).doubleValue() < array[particleA.getSelectedIndex()].restMass())
                                energyA.setText(Double.toString(array[particleA.getSelectedIndex()].restMass()));
                } else if(evt.target.equals(energyB)) {
                        if(Double.valueOf(energyB.getText()).doubleValue() < array[particleB.getSelectedIndex()].restMass())
                                energyB.setText(Double.toString(array[particleB.getSelectedIndex()].restMass()));
                }
                return true;
        }
        public QuantumParticle getParticleA() {
                QuantumParticle temp;
                try {
                        temp=(QuantumParticle) array[particleA.getSelectedIndex()].getClass().newInstance();
                } catch(Exception e) {
                        temp=null;
                }
                double energy=Double.valueOf(energyA.getText()).doubleValue();
                double mass=temp.restMass();
                temp.momentum=new ContraVector(energy,Math.sqrt(energy*energy-mass*mass),0.0,0.0);
                return temp;
        }
        public QuantumParticle getParticleB() {
                QuantumParticle temp;
                if(particleB.isEnabled()) {
                        try {
                                temp=(QuantumParticle) array[particleB.getSelectedIndex()].getClass().newInstance();
                        } catch(Exception e) {
                                temp=null;
                        }
                        double energy=Double.valueOf(energyB.getText()).doubleValue();
                        double mass=temp.restMass();
                        temp.momentum=new ContraVector(energy,-Math.sqrt(energy*energy-mass*mass),0.0,0.0);
                } else
                        temp=null;
                return temp;
        }
}


/**
* ResultsDialog.
* Displays the results of an experiment.
*
* @version 1.0
* @author Mark Hale
*/
class ResultsDialog extends Dialog {
        public ResultsDialog(Frame f,QuantumParticle in[],QuantumParticle out[],String interact) {
                super(f,"Results",false);
                resize(250,300);
                setLayout(new GridLayout(9,1));
                if(in[1]==null)
                        add(new Label(in[0].toString()+"------>"));
                else
                        add(new Label(in[0].toString()+" --->  <--- "+in[1].toString()));
                add(new Label(out[0].toString()));
                add(new Label("Energy = "+Double.toString(out[0].momentum.getComponent(0))));
                add(new Label("Momentum = "+Double.toString(out[0].momentum.getComponent(1))));
                add(new Label(out[1].toString()));
                add(new Label("Energy = "+Double.toString(out[1].momentum.getComponent(0))));
                add(new Label("Momentum = "+Double.toString(out[1].momentum.getComponent(1))));
                add(new Label("Interaction was the "+interact));
                add(new Button("OK"));
        }
        public boolean handleEvent(Event evt) {
                if(evt.id==Event.WINDOW_DESTROY || evt.target instanceof Button) {
                        hide();
                        dispose();
                        return true;
                } else {
                        return super.handleEvent(evt);
                }
        }
}

/**
* CMframe.
* Defines a class representing the Centre of Mass frame.
*
* @version 1.0
* @author Mark Hale
*/
final class CMframe {
        /**
        * Total energy.
        */
        private double energy;
        /**
        * Quantum number totals.
        */
        private int B,I,Iz,Le,Lm,Lt,Q,S;
        /**
        * The relative velocity between the Centre
        * of Mass frame and the lab frame.
        */
        private double relCMvel;
        /**
        * Interaction.
        */
        private int interaction;
        /**
        * Interaction codes.
        */
        private final static int STRONG=1,WEAK=2,EM=3;
        /**
        * Constructor.
        */
        public CMframe() {
                energy=B=I=Iz=Le=Lm=Lt=Q=S=0;
        }
        /**
        * Checks whether Particles a and b obey the
        * conservation laws.
        */
        public boolean conserve(QuantumParticle a,QuantumParticle b) {
                double Ei,Ej,pi,pj;
        // work out interaction type (also check conservation of quantum numbers)
                if(a.charge()+b.charge()==Q && a.eLeptonQN()+b.eLeptonQN()==Le &&
                        a.muLeptonQN()+b.muLeptonQN()==Lm && a.tauLeptonQN()+b.tauLeptonQN()==Lt &&
                        a.baryonQN()+b.baryonQN()==B && Math.abs(S-a.strangeQN()-b.strangeQN())<=1) {
                        if(a.strangeQN()+b.strangeQN()==S && a.isospinZ()+b.isospinZ()==Iz) {
                                if(a.isospin()+b.isospin()==I) {
                                        // Strong Interaction
                                        interaction=STRONG;
                                } else {
                                        // EM Interaction
                                        interaction=EM;
                                }
                        } else {
                                // Weak Interaction
                                interaction=WEAK;
                        }
                } else {
                        interaction=0;
                }
        // check quantum numbers are conserved
                if(interaction>0) {
        // Convert to keV for better accuracy
                        double massA=1000*a.restMass();
                        double massB=1000*b.restMass();
        // work out Ei,Ej,pi,pj
                        Ei=(energy*energy-massB*massB+massA*massA)/(2*energy);
                        Ej=(energy*energy-massA*massA+massB*massB)/(2*energy);
                        pi=(energy*energy*energy*energy +
                                massA*massA*massA*massA +
                                massB*massB*massB*massB -
                                2*massA*massA*energy*energy -
                                2*massB*massB*energy*energy -
                                2*massA*massA*massB*massB);
        // check consistency
                        if(pi>=0.0 && Ei>=0.0 && Ej>=0.0) {
                                pi=Math.sqrt(pi)/(2*energy);
        // Convert back to MeV and update E & p values
                                pi/=1000;
                                a.momentum=new ContraVector(Ei/1000,pi,0.0,0.0);
                                b.momentum=new ContraVector(Ej/1000,-pi,0.0,0.0);
        // Perform Lorentz Transform
                                RelativityMath.lorentzTransform(a,-relCMvel);
                                RelativityMath.lorentzTransform(b,-relCMvel);
                                return true;
                        }
                }
                return false;
        }
        public void addParticle(QuantumParticle qp) {
                B+=qp.baryonQN();
                I+=qp.isospin();
                Iz+=qp.isospinZ();
                Le+=qp.eLeptonQN();
                Lm+=qp.muLeptonQN();
                Lt+=qp.tauLeptonQN();
                Q+=qp.charge();
                S+=qp.strangeQN();
                RelativityMath.lorentzTransform(qp,relCMvel);
                energy+=qp.momentum.getComponent(0)*1000;       //store as keV for better accuracy
        }
        /**
        * Set the relative centre of mass velocity.
        */
        public void setRelCMvel(QuantumParticle a,QuantumParticle b) {
                if(b==null)
                        relCMvel=a.momentum.getComponent(1)/a.momentum.getComponent(0);
                else
                        relCMvel=(a.momentum.getComponent(1)+b.momentum.getComponent(1))/(a.momentum.getComponent(0)+b.momentum.getComponent(0));
        }
        /**
        * Returns the type of interaction that took place.
        */
        public String interact() {
                switch(interaction) {
                        case STRONG : return new String("Strong Force");
                        case WEAK : return new String("Weak Force");
                        case EM : return new String("Electromagnetic Force");
                        default : return new String("Unknown - error");
                }
        }
}
