package JSci.maths;

/**
* The Fourier math library.
* This class cannot be subclassed or instantiated because all methods are static.
* @version 0.2
* @author Mark Hale
*/
public final class FourierMath extends Object {
        static {System.loadLibrary("JSci");}
        private FourierMath() {}

        /**
        * Fourier transform.
        * @author Don Cross
        */
        public static Complex[] transform(Complex data[]) {
                Complex answer[]=new Complex[data.length];
                int numBits;
                int i,j,k,n;
                int blockSize,blockEnd;
                double angleNumerator=2.0*Math.PI;
                double deltaAngle;
                double alpha,beta; // used in recurrence relation
                Complex tmp,angle;

                if(!isPowerOf2(data.length))
                        throw new IllegalArgumentException("The number of samples must be a power of 2.");

                numBits=numberOfBitsNeeded(data.length);
// Simultaneous data copy and bit-reversal ordering into output
                for(i=0;i<data.length;i++) {
                        j=reverseBits(i,numBits);
                        answer[j]=data[i];
                }
// FFT
                blockEnd=1;
                for(blockSize=2;blockSize<=data.length;blockSize<<=1) {
                        deltaAngle=angleNumerator/blockSize;
                        alpha=Math.sin(0.5*deltaAngle);
                        alpha=2.0*alpha*alpha;
                        beta=Math.sin(deltaAngle);
                        for(i=0;i<data.length;i+=blockSize) {
                                angle=Complex.ONE;
                                for(j=i,n=0;n<blockEnd;j++,n++) {
                                        k=j+blockEnd;
                                        tmp=ComplexMath.multiply(angle,answer[k]);
                                        answer[k]=ComplexMath.subtract(answer[j],tmp);
                                        answer[j]=ComplexMath.add(answer[j],tmp);
                                        tmp=new Complex(alpha*angle.real()+beta*angle.imag(),
                                                alpha*angle.imag()-beta*angle.real());
                                        angle=ComplexMath.subtract(angle,tmp);
                                }
                        }
                        blockEnd=blockSize;
                }
                return answer;
        }
        /**
        * Inverse Fourier transform.
        * @author Don Cross
        */
        public static Complex[] inverseTransform(Complex data[]) {
                Complex answer[]=new Complex[data.length];
                int numBits;
                int i,j,k,n;
                int blockSize,blockEnd;
                double angleNumerator=-2.0*Math.PI;
                double deltaAngle;
                double alpha,beta; // used in recurrence relation
                Complex tmp,angle;

                if(!isPowerOf2(data.length))
                        throw new IllegalArgumentException("Data length must be a power of 2.");

                numBits=numberOfBitsNeeded(data.length);
// Simultaneous data copy and bit-reversal ordering into output
                for(i=0;i<data.length;i++) {
                        j=reverseBits(i,numBits);
                        answer[j]=data[i];
                }
// FFT
                blockEnd=1;
                for(blockSize=2;blockSize<=data.length;blockSize<<=1) {
                        deltaAngle=angleNumerator/blockSize;
                        alpha=Math.sin(0.5*deltaAngle);
                        alpha=2.0*alpha*alpha;
                        beta=Math.sin(deltaAngle);
                        for(i=0;i<data.length;i+=blockSize) {
                                angle=Complex.ONE;
                                for(j=i,n=0;n<blockEnd;j++,n++) {
                                        k=j+blockEnd;
                                        tmp=ComplexMath.multiply(angle,answer[k]);
                                        answer[k]=ComplexMath.subtract(answer[j],tmp);
                                        answer[j]=ComplexMath.add(answer[j],tmp);
                                        tmp=new Complex(alpha*angle.real()+beta*angle.imag(),
                                                alpha*angle.imag()-beta*angle.real());
                                        angle=ComplexMath.subtract(angle,tmp);
                                }
                        }
                        blockEnd=blockSize;
                }
// Normalize
                double denom=data.length;
                for(i=0;i<data.length;i++)
                        answer[i]=ComplexMath.divide(answer[i],denom);
                return answer;
        }

        /**
        * Returns true if x is a power of 2.
        * @author Don Cross
        */
        private native static boolean isPowerOf2(int x);
        /**
        * Number of bits needed.
        * @author Don Cross
        */
        private native static int numberOfBitsNeeded(int pwrOf2);
        /**
        * Reverse bits.
        * @author Don Cross
        */
        private native static int reverseBits(int index,int numBits);

        /**
        * Sorts the output from the Fourier transfom methods into ascending component order.
        */
        public static Complex[] sort(Complex output[]) {
                Complex ret[]=new Complex[output.length];
                int Nby2=output.length/2;
                for(int i=0;i<Nby2;i++) {
                        ret[Nby2+i]=output[i];
                        ret[i]=output[Nby2+i];
                }
                return ret;
        }
}

