package JSci.maths;

/**
* The matrix math library.
* This class cannot be subclassed or instantiated because all methods are static.
* @version 1.5
* @author Mark Hale
*/
public final class MatrixMath extends Object {
        static {System.loadLibrary("JSci");}
        private MatrixMath() {}

// HERMITIAN ADJOINT

        /**
        * Returns the hermitian adjoint of a complex matrix.
        * @param m a complex matrix
        */
        public static ComplexMatrix hermitianAdjoint(ComplexMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                Complex array[][]=new Complex[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[j][i]=ComplexMath.conjugate(m.getElement(i,j));
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns the hermitian adjoint of a complex square matrix.
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix hermitianAdjoint(ComplexSquareMatrix m) {
                int mRow=m.rows();
                Complex array[][]=new Complex[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[j][i]=ComplexMath.conjugate(m.getElement(i,j));
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns the hermitian adjoint of a complex tridiagonal matrix.
        * @param m a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix hermitianAdjoint(ComplexTridiagonalMatrix m) {
                int mRow=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(mRow);
                ans.setElement(0,0,ComplexMath.conjugate(m.getElement(0,0)));
                ans.setElement(0,1,ComplexMath.conjugate(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,ComplexMath.conjugate(m.getElement(i-1,i)));
                        ans.setElement(i,i,ComplexMath.conjugate(m.getElement(i,i)));
                        ans.setElement(i,i+1,ComplexMath.conjugate(m.getElement(i+1,i)));
                }
                ans.setElement(mRow,mRow-1,ComplexMath.conjugate(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,ComplexMath.conjugate(m.getElement(mRow,mRow)));
                return ans;
        }

// COMPLEX CONJUGATE

        /**
        * Returns the complex conjugate of a complex matrix.
        * @param m a complex matrix
        */
        public static ComplexMatrix conjugate(ComplexMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                Complex array[][]=new Complex[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[i][j]=ComplexMath.conjugate(m.getElement(i,j));
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns the complex conjugate of a complex square matrix.
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix conjugate(ComplexSquareMatrix m) {
                int mRow=m.rows();
                Complex array[][]=new Complex[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[i][j]=ComplexMath.conjugate(m.getElement(i,j));
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns the complex conjugate of a complex tridiagonal matrix.
        * @param m a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix conjugate(ComplexTridiagonalMatrix m) {
                int mRow=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(mRow);
                ans.setElement(0,0,ComplexMath.conjugate(m.getElement(0,0)));
                ans.setElement(1,0,ComplexMath.conjugate(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i-1,i,ComplexMath.conjugate(m.getElement(i-1,i)));
                        ans.setElement(i,i,ComplexMath.conjugate(m.getElement(i,i)));
                        ans.setElement(i+1,i,ComplexMath.conjugate(m.getElement(i+1,i)));
                }
                ans.setElement(mRow-1,mRow,ComplexMath.conjugate(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,ComplexMath.conjugate(m.getElement(mRow,mRow)));
                return ans;
        }
        /**
        * Returns the complex conjugate of a complex diagonal matrix.
        * @param m a complex diagonal matrix
        */
        public static ComplexDiagonalMatrix conjugate(ComplexDiagonalMatrix m) {
                int mRow=m.rows();
                Complex array[]=new Complex[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=ComplexMath.conjugate(m.getElement(i,i));
                return new ComplexDiagonalMatrix(array);
        }

// ADDITION
 
        /**
        * Returns the addition of two matrices.
        * @param a an integer matrix
        * @param b an integer matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerMatrix add(IntegerMatrix a, IntegerMatrix b) {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        int array[][]=new int[aRow][aCol];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        array[i][j]=a.getElement(i,j)+b.getElement(i,j);
                        }
                        return new IntegerMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two square matrices.
        * @param a an integer square matrix
        * @param b an integer square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerSquareMatrix add(IntegerSquareMatrix a, IntegerSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[][]=new int[aRow][aRow];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        array[i][j]=a.getElement(i,j)+b.getElement(i,j);
                        }
                        return new IntegerSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two tridiagonal matrices.
        * @param a an integer tridiagonal matrix
        * @param b an integer tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerTridiagonalMatrix add(IntegerTridiagonalMatrix a,IntegerTridiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        IntegerTridiagonalMatrix ans=new IntegerTridiagonalMatrix(aRow);
                        ans.setElement(0,0,a.getElement(0,0)+b.getElement(0,0));
                        ans.setElement(1,0,a.getElement(1,0)+b.getElement(1,0));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,a.getElement(i-1,i)+b.getElement(i-1,i));
                                ans.setElement(i,i,a.getElement(i-1,i)+b.getElement(i,i));
                                ans.setElement(i+1,i,a.getElement(i+1,i)+b.getElement(i+1,i));
                        }
                        ans.setElement(aRow-1,aRow,a.getElement(aRow-1,aRow)+b.getElement(aRow-1,aRow));
                        ans.setElement(aRow,aRow,a.getElement(aRow,aRow)+b.getElement(aRow,aRow));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two diagonal matrices.
        * @param a an integer diagonal matrix
        * @param b an integer diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerDiagonalMatrix add(IntegerDiagonalMatrix a, IntegerDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[]=new int[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)+b.getElement(i,i);
                        return new IntegerDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the addition of two matrices.
        * @param a a double matrix
        * @param b a double matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleMatrix add(DoubleMatrix a, DoubleMatrix b) {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        double array[][]=new double[aRow][aCol];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        array[i][j]=a.getElement(i,j)+b.getElement(i,j);
                        }
                        return new DoubleMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two square matrices.
        * @param a a double square matrix
        * @param b a double square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleSquareMatrix add(DoubleSquareMatrix a, DoubleSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[][]=new double[aRow][aRow];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        array[i][j]=a.getElement(i,j)+b.getElement(i,j);
                        }
                        return new DoubleSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two tridiagonal matrices.
        * @param a a double tridiagonal matrix
        * @param b a double tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleTridiagonalMatrix add(DoubleTridiagonalMatrix a,DoubleTridiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(aRow);
                        ans.setElement(0,0,a.getElement(0,0)+b.getElement(0,0));
                        ans.setElement(1,0,a.getElement(1,0)+b.getElement(1,0));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,a.getElement(i-1,i)+b.getElement(i-1,i));
                                ans.setElement(i,i,a.getElement(i-1,i)+b.getElement(i,i));
                                ans.setElement(i+1,i,a.getElement(i+1,i)+b.getElement(i+1,i));
                        }
                        ans.setElement(aRow-1,aRow,a.getElement(aRow-1,aRow)+b.getElement(aRow-1,aRow));
                        ans.setElement(aRow,aRow,a.getElement(aRow,aRow)+b.getElement(aRow,aRow));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two diagonal matrices.
        * @param a a double diagonal matrix
        * @param b a double diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleDiagonalMatrix add(DoubleDiagonalMatrix a, DoubleDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[]=new double[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)+b.getElement(i,i);
                        return new DoubleDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the addition of two matrices.
        * @param a a complex matrix
        * @param b a complex matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexMatrix add(ComplexMatrix a, ComplexMatrix b) {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        Complex array[][]=new Complex[aRow][aCol];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        array[i][j]=ComplexMath.add(a.getElement(i,j), b.getElement(i,j));
                        }
                        return new ComplexMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two square matrices.
        * @param a a complex square matrix
        * @param b a complex square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexSquareMatrix add(ComplexSquareMatrix a, ComplexSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[][]=new Complex[aRow][aRow];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        array[i][j]=ComplexMath.add(a.getElement(i,j), b.getElement(i,j));
                        }
                        return new ComplexSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two tridiagonal matrices.
        * @param a a complex tridiagonal matrix
        * @param b a complex tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexTridiagonalMatrix add(ComplexTridiagonalMatrix a,ComplexTridiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(aRow);
                        ans.setElement(0,0,ComplexMath.add(a.getElement(0,0),b.getElement(0,0)));
                        ans.setElement(1,0,ComplexMath.add(a.getElement(1,0),b.getElement(1,0)));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,ComplexMath.add(a.getElement(i-1,i),b.getElement(i-1,i)));
                                ans.setElement(i,i,ComplexMath.add(a.getElement(i-1,i),b.getElement(i,i)));
                                ans.setElement(i+1,i,ComplexMath.add(a.getElement(i+1,i),b.getElement(i+1,i)));
                        }
                        ans.setElement(aRow-1,aRow,ComplexMath.add(a.getElement(aRow-1,aRow),b.getElement(aRow-1,aRow)));
                        ans.setElement(aRow,aRow,ComplexMath.add(a.getElement(aRow,aRow),b.getElement(aRow,aRow)));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the addition of two diagonal matrices.
        * @param a a complex diagonal matrix
        * @param b a complex diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexDiagonalMatrix add(ComplexDiagonalMatrix a, ComplexDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[]=new Complex[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=ComplexMath.add(a.getElement(i,i),b.getElement(i,i));
                        return new ComplexDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

// SUBTRACTION

        /**
        * Returns the subtraction of two matrices.
        * @param a an integer matrix
        * @param b an integer matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerMatrix subtract(IntegerMatrix a, IntegerMatrix b) {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        int array[][]=new int[aRow][aCol];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        array[i][j]=a.getElement(i,j)-b.getElement(i,j);
                        }
                        return new IntegerMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two square matrices.
        * @param a an integer square matrix
        * @param b an integer square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerSquareMatrix subtract(IntegerSquareMatrix a, IntegerSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[][]=new int[aRow][aRow];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        array[i][j]=a.getElement(i,j)-b.getElement(i,j);
                        }
                        return new IntegerSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two tridiagonal matrices.
        * @param a an integer tridiagonal matrix
        * @param b an integer tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerTridiagonalMatrix subtract(IntegerTridiagonalMatrix a,IntegerTridiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        IntegerTridiagonalMatrix ans=new IntegerTridiagonalMatrix(aRow);
                        ans.setElement(0,0,a.getElement(0,0)-b.getElement(0,0));
                        ans.setElement(1,0,a.getElement(1,0)-b.getElement(1,0));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,a.getElement(i-1,i)-b.getElement(i-1,i));
                                ans.setElement(i,i,a.getElement(i-1,i)-b.getElement(i,i));
                                ans.setElement(i+1,i,a.getElement(i+1,i)-b.getElement(i+1,i));
                        }
                        ans.setElement(aRow-1,aRow,a.getElement(aRow-1,aRow)-b.getElement(aRow-1,aRow));
                        ans.setElement(aRow,aRow,a.getElement(aRow,aRow)-b.getElement(aRow,aRow));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two diagonal matrices.
        * @param a an integer diagonal matrix
        * @param b an integer diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerDiagonalMatrix subtract(IntegerDiagonalMatrix a, IntegerDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[]=new int[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)-b.getElement(i,i);
                        return new IntegerDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the subtraction of two matrices.
        * @param a a double matrix
        * @param b a double matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleMatrix subtract(DoubleMatrix a, DoubleMatrix b) {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        double array[][]=new double[aRow][aCol];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        array[i][j]=a.getElement(i,j)-b.getElement(i,j);
                        }
                        return new DoubleMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two square matrices.
        * @param a a double square matrix
        * @param b a double square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleSquareMatrix subtract(DoubleSquareMatrix a, DoubleSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[][]=new double[aRow][aRow];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        array[i][j]=a.getElement(i,j)-b.getElement(i,j);
                        }
                        return new DoubleSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two tridiagonal matrices.
        * @param a a double tridiagonal matrix
        * @param b a double tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleTridiagonalMatrix subtract(DoubleTridiagonalMatrix a,DoubleTridiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(aRow);
                        ans.setElement(0,0,a.getElement(0,0)-b.getElement(0,0));
                        ans.setElement(1,0,a.getElement(1,0)-b.getElement(1,0));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,a.getElement(i-1,i)-b.getElement(i-1,i));
                                ans.setElement(i,i,a.getElement(i-1,i)-b.getElement(i,i));
                                ans.setElement(i+1,i,a.getElement(i+1,i)-b.getElement(i+1,i));
                        }
                        ans.setElement(aRow-1,aRow,a.getElement(aRow-1,aRow)-b.getElement(aRow-1,aRow));
                        ans.setElement(aRow,aRow,a.getElement(aRow,aRow)-b.getElement(aRow,aRow));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two diagonal matrices.
        * @param a a double diagonal matrix
        * @param b a double diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleDiagonalMatrix subtract(DoubleDiagonalMatrix a, DoubleDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[]=new double[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)-b.getElement(i,i);
                        return new DoubleDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the subtraction of two matrices.
        * @param a a complex matrix
        * @param b a complex matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexMatrix subtract(ComplexMatrix a, ComplexMatrix b) {
                int aRow=a.rows(),aCol=a.columns();
                if(aRow==b.rows() && aCol==b.columns()) {
                        Complex array[][]=new Complex[aRow][aCol];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aCol;j++)
                                        array[i][j]=ComplexMath.subtract(a.getElement(i,j), b.getElement(i,j));
                        }
                        return new ComplexMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two square matrices.
        * @param a a complex square matrix
        * @param b a complex square matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexSquareMatrix subtract(ComplexSquareMatrix a, ComplexSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[][]=new Complex[aRow][aRow];
                        for(int j,i=0;i<aRow;i++) {
                                for(j=0;j<aRow;j++)
                                        array[i][j]=ComplexMath.subtract(a.getElement(i,j), b.getElement(i,j));
                        }
                        return new ComplexSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two tridiagonal matrices.
        * @param a a complex tridiagonal matrix
        * @param b a complex tridiagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexTridiagonalMatrix subtract(ComplexTridiagonalMatrix a,ComplexTridiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(aRow);
                        ans.setElement(0,0,ComplexMath.subtract(a.getElement(0,0),b.getElement(0,0)));
                        ans.setElement(1,0,ComplexMath.subtract(a.getElement(1,0),b.getElement(1,0)));
                        aRow--;
                        for(int i=1;i<aRow;i++) {
                                ans.setElement(i-1,i,ComplexMath.subtract(a.getElement(i-1,i),b.getElement(i-1,i)));
                                ans.setElement(i,i,ComplexMath.subtract(a.getElement(i-1,i),b.getElement(i,i)));
                                ans.setElement(i+1,i,ComplexMath.subtract(a.getElement(i+1,i),b.getElement(i+1,i)));
                        }
                        ans.setElement(aRow-1,aRow,ComplexMath.subtract(a.getElement(aRow-1,aRow),b.getElement(aRow-1,aRow)));
                        ans.setElement(aRow,aRow,ComplexMath.subtract(a.getElement(aRow,aRow),b.getElement(aRow,aRow)));
                        return ans;
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }
        /**
        * Returns the subtraction of two diagonal matrices.
        * @param a a complex diagonal matrix
        * @param b a complex diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexDiagonalMatrix subtract(ComplexDiagonalMatrix a, ComplexDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[]=new Complex[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=ComplexMath.subtract(a.getElement(i,i),b.getElement(i,i));
                        return new ComplexDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

// MATRIX OPERATION

        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m an integer matrix
        * @param a an integer vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static IntegerVector multiply(IntegerMatrix m, IntegerVector a) {
                int mRow=m.rows(),mCol=m.columns();
                if(mCol==a.dimension()) {
                        int array[]=new int[mRow];
                        for(int j,i=0;i<mRow;i++) {
                                array[i]=0;
                                for(j=0;j<mCol;j++)
                                        array[i]+=m.getElement(i,j)*a.getComponent(j);
                        }
                        return new IntegerVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }
        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m an integer diagonal matrix
        * @param a an integer vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static IntegerVector multiply(IntegerDiagonalMatrix m, IntegerVector a) {
                int mRow=m.rows();
                if(mRow==a.dimension()) {
                        int array[]=new int[mRow];
                        for(int i=0;i<mRow;i++)
                                array[i]=m.getElement(i,i)*a.getComponent(i);
                        return new IntegerVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }

        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m a double matrix
        * @param a a double vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static DoubleVector multiply(DoubleMatrix m, DoubleVector a) {
                int mRow=m.rows(),mCol=m.columns();
                if(mCol==a.dimension()) {
                        double array[]=new double[mRow];
                        for(int j,i=0;i<mRow;i++) {
                                array[i]=0.0;
                                for(j=0;j<mCol;j++)
                                        array[i]+=m.getElement(i,j)*a.getComponent(j);
                        }
                        return new DoubleVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }
        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m a double diagonal matrix
        * @param a a double vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static DoubleVector multiply(DoubleDiagonalMatrix m, DoubleVector a) {
                int mRow=m.rows();
                if(mRow==a.dimension()) {
                        double array[]=new double[mRow];
                        for(int i=0;i<mRow;i++)
                                array[i]=m.getElement(i,i)*a.getComponent(i);
                        return new DoubleVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }

        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m a complex matrix
        * @param a a complex vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static ComplexVector multiply(ComplexMatrix m, ComplexVector a) {
                int mRow=m.rows(),mCol=m.columns();
                if(mCol==a.dimension()) {
                        Complex array[]=new Complex[mRow];
                        for(int j,i=0;i<mRow;i++) {
                                array[i]=Complex.ZERO;
                                for(j=0;j<mCol;j++)
                                        array[i]=ComplexMath.add(array[i], ComplexMath.multiply(m.getElement(i,j), a.getComponent(j)));
                        }
                        return new ComplexVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }
        /**
        * Returns the multiplication of a vector by a matrix.
        * @param m a complex diagonal matrix
        * @param a a complex vector
        * @exception DimensionException If the matrix and vector are incompatible.
        */
        public static ComplexVector multiply(ComplexDiagonalMatrix m, ComplexVector a) {
                int mRow=m.rows();
                if(mRow==a.dimension()) {
                        Complex array[]=new Complex[mRow];
                        for(int i=0;i<mRow;i++)
                                array[i]=ComplexMath.multiply(m.getElement(i,i),a.getComponent(i));
                        return new ComplexVector(array);
                } else
                        throw new DimensionException("Matrix and vector are incompatible.");
        }

// SCALAR MULTIPLICATION

        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x an integer
        * @param m an integer matrix
        */
        public static IntegerMatrix scalarMultiply(int x, IntegerMatrix m) {
                int row=m.rows(),col=m.columns();
                int array[][]=new int[row][col];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<col;j++)
                                array[i][j]=x*m.getElement(i,j);
                }
                return new IntegerMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x an integer
        * @param m an integer square matrix
        */
        public static IntegerSquareMatrix scalarMultiply(int x, IntegerSquareMatrix m) {
                int row=m.rows();
                int array[][]=new int[row][row];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<row;j++)
                                array[i][j]=x*m.getElement(i,j);
                }
                return new IntegerSquareMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x an integer
        * @param m an integer tridiagonal matrix
        */
        public static IntegerTridiagonalMatrix scalarMultiply(int x, IntegerTridiagonalMatrix m) {
                int row=m.rows();
                IntegerTridiagonalMatrix ans=new IntegerTridiagonalMatrix(row);
                ans.setElement(0,0,x*m.getElement(0,0));
                ans.setElement(0,1,x*m.getElement(1,0));
                row--;
                for(int i=1;i<row;i++) {
                        ans.setElement(i,i-1,x*m.getElement(i-1,i));
                        ans.setElement(i,i,x*m.getElement(i,i));
                        ans.setElement(i,i+1,x*m.getElement(i+1,i));
                }
                ans.setElement(row,row-1,x*m.getElement(row-1,row));
                ans.setElement(row,row,x*m.getElement(row,row));
                return ans;
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x an integer
        * @param m an integer diagonal matrix
        */
        public static IntegerDiagonalMatrix scalarMultiply(int x, IntegerDiagonalMatrix m) {
                int row=m.rows();
                int array[]=new int[row];
                for(int i=0;i<row;i++) {
                        array[i]=x*m.getElement(i,i);
                }
                return new IntegerDiagonalMatrix(array);
        }

        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x a double
        * @param m a double matrix
        */
        public static DoubleMatrix scalarMultiply(double x, DoubleMatrix m) {
                int row=m.rows(),col=m.columns();
                double array[][]=new double[row][col];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<col;j++)
                                array[i][j]=x*m.getElement(i,j);
                }
                return new DoubleMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x a double
        * @param m a double square matrix
        */
        public static DoubleSquareMatrix scalarMultiply(double x, DoubleSquareMatrix m) {
                int row=m.rows();
                double array[][]=new double[row][row];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<row;j++)
                                array[i][j]=x*m.getElement(i,j);
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x a double
        * @param m a double tridiagonal matrix
        */
        public static DoubleTridiagonalMatrix scalarMultiply(double x, DoubleTridiagonalMatrix m) {
                int row=m.rows();
                DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(row);
                ans.setElement(0,0,x*m.getElement(0,0));
                ans.setElement(0,1,x*m.getElement(1,0));
                row--;
                for(int i=1;i<row;i++) {
                        ans.setElement(i,i-1,x*m.getElement(i-1,i));
                        ans.setElement(i,i,x*m.getElement(i,i));
                        ans.setElement(i,i+1,x*m.getElement(i+1,i));
                }
                ans.setElement(row,row-1,x*m.getElement(row-1,row));
                ans.setElement(row,row,x*m.getElement(row,row));
                return ans;
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param x a double
        * @param m a double diagonal matrix
        */
        public static DoubleDiagonalMatrix scalarMultiply(double x, DoubleDiagonalMatrix m) {
                int row=m.rows();
                double array[]=new double[row];
                for(int i=0;i<row;i++) {
                        array[i]=x*m.getElement(i,i);
                }
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param z a complex number
        * @param m a complex matrix
        */
        public static ComplexMatrix scalarMultiply(Complex z, ComplexMatrix m) {
                int row=m.rows(),col=m.columns();
                Complex array[][]=new Complex[row][col];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<col;j++)
                                array[i][j]=ComplexMath.multiply(z,m.getElement(i,j));
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param z a complex number
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix scalarMultiply(Complex z, ComplexSquareMatrix m) {
                int row=m.rows();
                Complex array[][]=new Complex[row][row];
                for(int j,i=0;i<row;i++) {
                        for(j=0;j<row;j++)
                                array[i][j]=ComplexMath.multiply(z,m.getElement(i,j));
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param z a complex number
        * @param m a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix scalarMultiply(Complex z, ComplexTridiagonalMatrix m) {
                int row=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(row);
                ans.setElement(0,0,ComplexMath.multiply(z,m.getElement(0,0)));
                ans.setElement(0,1,ComplexMath.multiply(z,m.getElement(1,0)));
                row--;
                for(int i=1;i<row;i++) {
                        ans.setElement(i,i-1,ComplexMath.multiply(z,m.getElement(i-1,i)));
                        ans.setElement(i,i,ComplexMath.multiply(z,m.getElement(i,i)));
                        ans.setElement(i,i+1,ComplexMath.multiply(z,m.getElement(i+1,i)));
                }
                ans.setElement(row,row-1,ComplexMath.multiply(z,m.getElement(row-1,row)));
                ans.setElement(row,row,ComplexMath.multiply(z,m.getElement(row,row)));
                return ans;
        }
        /**
        * Returns the multiplication of a matrix by a scalar.
        * @param z a complex number
        * @param m a complex diagonal matrix
        */
        public static ComplexDiagonalMatrix scalarMultiply(Complex z, ComplexDiagonalMatrix m) {
                int row=m.rows();
                Complex array[]=new Complex[row];
                for(int i=0;i<row;i++) {
                        array[i]=ComplexMath.multiply(z,m.getElement(i,i));
                }
                return new ComplexDiagonalMatrix(array);
        }

// MATRIX MULTIPLICATION

        /**
        * Returns the multiplication of two matrices.
        * @param a an integer matrix
        * @param b an integer matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static IntegerMatrix multiply(IntegerMatrix a, IntegerMatrix b) {
                int aRow=a.rows(),bRow=b.rows(),bCol=b.columns();
                if(a.columns()==bRow) {
                        int n,k;
                        int array[][]=new int[aRow][bCol];
                        for(int j=0;j<aRow;j++) {
                                for(k=0;k<bCol;k++) {
                                        array[j][k]=0;
                                        for(n=0;n<bRow;n++)
                                                array[j][k]+=a.getElement(j,n)*b.getElement(n,k);
                                }
                        }
                        return new IntegerMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two square matrices.
        * @param a an integer square matrix
        * @param b an integer square matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static IntegerSquareMatrix multiply(IntegerSquareMatrix a, IntegerSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int n,k;
                        int array[][]=new int[aRow][aRow];
                        for(int j=0;j<aRow;j++) {
                                for(k=0;k<aRow;k++) {
                                        array[j][k]=0;
                                        for(n=0;n<aRow;n++)
                                                array[j][k]+=a.getElement(j,n)*b.getElement(n,k);
                                }
                        }
                        return new IntegerSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two diagonal matrices.
        * @param a an integer diagonal matrix
        * @param b an integer diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static IntegerDiagonalMatrix multiply(IntegerDiagonalMatrix a, IntegerDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int array[]=new int[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)*b.getElement(i,i);
                        return new IntegerDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the multiplication of two matrices.
        * @param a a double matrix
        * @param b a double matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static DoubleMatrix multiply(DoubleMatrix a, DoubleMatrix b) {
                int aRow=a.rows(),bRow=b.rows(),bCol=b.columns();
                if(a.columns()==bRow) {
                        int n,k;
                        double array[][]=new double[aRow][bCol];
                        for(int j=0;j<aRow;j++) {
                                for(k=0;k<bCol;k++) {
                                        array[j][k]=0.0;
                                        for(n=0;n<bRow;n++)
                                                array[j][k]+=a.getElement(j,n)*b.getElement(n,k);
                                }
                        }
                        return new DoubleMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two square matrices.
        * @param a a double square matrix
        * @param b a double square matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static DoubleSquareMatrix multiply(DoubleSquareMatrix a, DoubleSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int n,k;
                        double array[][]=new double[aRow][aRow];
                        for(int j=0;j<aRow;j++) {
                                for(k=0;k<aRow;k++) {
                                        array[j][k]=0.0;
                                        for(n=0;n<aRow;n++)
                                                array[j][k]+=a.getElement(j,n)*b.getElement(n,k);
                                }
                        }
                        return new DoubleSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two diagonal matrices.
        * @param a a double diagonal matrix
        * @param b a double diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static DoubleDiagonalMatrix multiply(DoubleDiagonalMatrix a, DoubleDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        double array[]=new double[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=a.getElement(i,i)*b.getElement(i,i);
                        return new DoubleDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

        /**
        * Returns the multiplication of two matrices.
        * @param a a complex matrix
        * @param b a complex matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static ComplexMatrix multiply(ComplexMatrix a, ComplexMatrix b) {
                int aRow=a.rows(),bRow=b.rows(),bCol=b.columns();
                if(a.columns()==bRow) {
                        int n,k;
                        Complex array[][]=new Complex[aRow][bCol];
                        for(int j=0;j<aRow;j++) {
                                for(k=0;k<bCol;k++) {
                                        array[j][k]=Complex.ZERO;
                                        for(n=0;n<bRow;n++)
                                                array[j][k]=ComplexMath.add(array[j][k], ComplexMath.multiply(a.getElement(j,n), b.getElement(n,k)));
                                }
                        }
                        return new ComplexMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two square matrices.
        * @param a a complex square matrix
        * @param b a complex square matrix
        * @exception MatrixDimensionException If the matrices are incompatible.
        */
        public static ComplexSquareMatrix multiply(ComplexSquareMatrix a, ComplexSquareMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        int n,k;
                        Complex array[][]=new Complex[aRow][aRow];
                        for(int j=0;j<aRow;j++) {
                                for(k=0;k<aRow;k++) {
                                        array[j][k]=Complex.ZERO;
                                        for(n=0;n<aRow;n++)
                                                array[j][k]=ComplexMath.add(array[j][k], ComplexMath.multiply(a.getElement(j,n), b.getElement(n,k)));
                                }
                        }
                        return new ComplexSquareMatrix(array);
                } else
                        throw new MatrixDimensionException("Incompatible matrices.");
        }
        /**
        * Returns the multiplication of two diagonal matrices.
        * @param a a complex diagonal matrix
        * @param b a complex diagonal matrix
        * @exception MatrixDimensionException If the matrices are different sizes.
        */
        public static ComplexDiagonalMatrix multiply(ComplexDiagonalMatrix a, ComplexDiagonalMatrix b) {
                int aRow=a.rows();
                if(aRow==b.rows()) {
                        Complex array[]=new Complex[aRow];
                        for(int i=0;i<aRow;i++)
                                array[i]=ComplexMath.multiply(a.getElement(i,i),b.getElement(i,i));
                        return new ComplexDiagonalMatrix(array);
                } else
                        throw new MatrixDimensionException("Matrices are different sizes.");
        }

// TRANSPOSE

        /**
        * Returns the transpose of a matrix.
        * @param m an integer matrix
        */
        public static IntegerMatrix transpose(IntegerMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                int array[][]=new int[mCol][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new IntegerMatrix(array);
        }
        /**
        * Returns the transpose of a square matrix.
        * @param m an integer square matrix
        */
        public static IntegerSquareMatrix transpose(IntegerSquareMatrix m) {
                int mRow=m.rows();
                int array[][]=new int[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new IntegerSquareMatrix(array);
        }
        /**
        * Returns the transpose of a tridiagonal matrix.
        * @param m an integer tridiagonal matrix
        */
        public static IntegerTridiagonalMatrix transpose(IntegerTridiagonalMatrix m) {
                int mRow=m.rows();
                IntegerTridiagonalMatrix ans=new IntegerTridiagonalMatrix(mRow);
                ans.setElement(0,0,m.getElement(0,0));
                ans.setElement(0,1,m.getElement(1,0));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,m.getElement(i-1,i));
                        ans.setElement(i,i,m.getElement(i,i));
                        ans.setElement(i,i+1,m.getElement(i+1,i));
                }
                ans.setElement(mRow,mRow-1,m.getElement(mRow-1,mRow));
                ans.setElement(mRow,mRow,m.getElement(mRow,mRow));
                return ans;
        }

        /**
        * Returns the transpose of a matrix.
        * @param m a double matrix
        */
        public static DoubleMatrix transpose(DoubleMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                double array[][]=new double[mCol][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new DoubleMatrix(array);
        }
        /**
        * Returns the transpose of a square matrix.
        * @param m a double square matrix
        */
        public static DoubleSquareMatrix transpose(DoubleSquareMatrix m) {
                int mRow=m.rows();
                double array[][]=new double[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Returns the transpose of a tridiagonal matrix.
        * @param m a double tridiagonal matrix
        */
        public static DoubleTridiagonalMatrix transpose(DoubleTridiagonalMatrix m) {
                int mRow=m.rows();
                DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(mRow);
                ans.setElement(0,0,m.getElement(0,0));
                ans.setElement(0,1,m.getElement(1,0));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,m.getElement(i-1,i));
                        ans.setElement(i,i,m.getElement(i,i));
                        ans.setElement(i,i+1,m.getElement(i+1,i));
                }
                ans.setElement(mRow,mRow-1,m.getElement(mRow-1,mRow));
                ans.setElement(mRow,mRow,m.getElement(mRow,mRow));
                return ans;
        }

        /**
        * Returns the transpose of a matrix.
        * @param m a complex matrix
        */
        public static ComplexMatrix transpose(ComplexMatrix m) {
                int mRow=m.rows(),mCol=m.columns();
                Complex array[][]=new Complex[mCol][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns the transpose of a square matrix.
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix transpose(ComplexSquareMatrix m) {
                int mRow=m.rows();
                Complex array[][]=new Complex[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[j][i]=m.getElement(i,j);
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Returns the transpose of a tridiagonal matrix.
        * @param m a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix transpose(ComplexTridiagonalMatrix m) {
                int mRow=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(mRow);
                ans.setElement(0,0,m.getElement(0,0));
                ans.setElement(0,1,m.getElement(1,0));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,m.getElement(i-1,i));
                        ans.setElement(i,i,m.getElement(i,i));
                        ans.setElement(i,i+1,m.getElement(i+1,i));
                }
                ans.setElement(mRow,mRow-1,m.getElement(mRow-1,mRow));
                ans.setElement(mRow,mRow,m.getElement(mRow,mRow));
                return ans;
        }

// INVERSE

        /**
        * Returns the inverse of a matrix.
        * @param m an integer square matrix
        */
        public static DoubleSquareMatrix inverse(IntegerSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                double tmpL,tmpU,array[][][]=new double[2][mRow][mRow];
                DoubleSquareMatrix lu[]=LUDecomposition(m);
                for(i=0;i<mRow;i++) {
                        array[0][i][i]=1/lu[0].getElement(i,i);
                        array[1][i][i]=1/lu[1].getElement(i,i);
                }
                for(i=0;i<mRow-1;i++) {
                        for(j=i+1;j<mRow;j++) {
                                tmpL=tmpU=0.0;
                                for(k=i;k<j;k++) {
                                        tmpL-=lu[0].getElement(j,k)*array[0][k][i];
                                        tmpU-=array[1][i][k]*lu[1].getElement(k,j);
                                }
                                array[0][j][i]=tmpL/lu[0].getElement(j,j);
                                array[1][i][j]=tmpU/lu[1].getElement(j,j);
                        }
                }
                return MatrixMath.multiply(new DoubleSquareMatrix(array[1]),new DoubleSquareMatrix(array[0]));
        }
        /**
        * Returns the inverse of a matrix.
        * @param m an integer diagonal matrix
        */
        public static DoubleDiagonalMatrix inverse(IntegerDiagonalMatrix m) {
                int mRow=m.rows();
                double array[]=new double[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=1/m.getElement(i,i);
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Returns the inverse of a matrix.
        * @param m a double square matrix
        */
        public static DoubleSquareMatrix inverse(DoubleSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                double tmpL,tmpU,array[][][]=new double[2][mRow][mRow];
                DoubleSquareMatrix lu[]=LUDecomposition(m);
                for(i=0;i<mRow;i++) {
                        array[0][i][i]=1/lu[0].getElement(i,i);
                        array[1][i][i]=1/lu[1].getElement(i,i);
                }
                for(i=0;i<mRow-1;i++) {
                        for(j=i+1;j<mRow;j++) {
                                tmpL=tmpU=0.0;
                                for(k=i;k<j;k++) {
                                        tmpL-=lu[0].getElement(j,k)*array[0][k][i];
                                        tmpU-=array[1][i][k]*lu[1].getElement(k,j);
                                }
                                array[0][j][i]=tmpL/lu[0].getElement(j,j);
                                array[1][i][j]=tmpU/lu[1].getElement(j,j);
                        }
                }
                return MatrixMath.multiply(new DoubleSquareMatrix(array[1]),new DoubleSquareMatrix(array[0]));
        }
        /**
        * Returns the inverse of a matrix.
        * @param m a double diagonal matrix
        */
        public static DoubleDiagonalMatrix inverse(DoubleDiagonalMatrix m) {
                int mRow=m.rows();
                double array[]=new double[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=1/m.getElement(i,i);
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Returns the inverse of a matrix.
        * @param m a complex square matrix
        */
        public static ComplexSquareMatrix inverse(ComplexSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                Complex tmpL,tmpU,array[][][]=new Complex[2][mRow][mRow];
                ComplexSquareMatrix lu[]=LUDecomposition(m);
                for(i=0;i<mRow;i++) {
                        array[0][i][i]=ComplexMath.divide(Complex.ONE,lu[0].getElement(i,i));
                        array[1][i][i]=ComplexMath.divide(Complex.ONE,lu[1].getElement(i,i));
                }
                for(i=0;i<mRow-1;i++) {
                        for(j=i+1;j<mRow;j++) {
                                tmpL=tmpU=Complex.ZERO;
                                for(k=i;k<j;k++) {
                                        tmpL=ComplexMath.subtract(tmpL,ComplexMath.multiply(lu[0].getElement(j,k),array[0][k][i]));
                                        tmpU=ComplexMath.subtract(tmpU,ComplexMath.multiply(array[1][i][k],lu[1].getElement(k,j)));
                                }
                                array[0][i][j]=Complex.ZERO;
                                array[0][j][i]=ComplexMath.divide(tmpL,lu[0].getElement(j,j));
                                array[1][j][i]=Complex.ZERO;
                                array[1][i][j]=ComplexMath.divide(tmpU,lu[1].getElement(j,j));
                        }
                }
                return MatrixMath.multiply(new ComplexSquareMatrix(array[1]),new ComplexSquareMatrix(array[0]));
        }
        /**
        * Returns the inverse of a matrix.
        * @param m a complex diagonal matrix
        */
        public static ComplexDiagonalMatrix inverse(ComplexDiagonalMatrix m) {
                int mRow=m.rows();
                Complex one=new Complex(1.0,0.0);
                Complex array[]=new Complex[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=ComplexMath.divide(one,m.getElement(i,i));
                return new ComplexDiagonalMatrix(array);
        }

// ELEMENT MAP

        /**
        * Applies a function on all the matrix elements.
        * @param m an integer matrix
        * @param f a user-defined function
        * @return a double matrix
        */
        public static DoubleMatrix mapElements(IntegerMatrix m, Mapping f) {
                int mRow=m.rows(),mCol=m.columns();
                double array[][]=new double[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new DoubleMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m an integer square matrix
        * @param f a user-defined function
        * @return a double square matrix
        */
        public static DoubleSquareMatrix mapElements(IntegerSquareMatrix m, Mapping f) {
                int mRow=m.rows();
                double array[][]=new double[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m an integer tridiagonal matrix
        * @param f a user-defined function
        * @return a double tridiagonal matrix
        */
        public static DoubleTridiagonalMatrix mapElements(IntegerTridiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(mRow);
                ans.setElement(0,0,f.map(m.getElement(0,0)));
                ans.setElement(0,1,f.map(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,f.map(m.getElement(i-1,i)));
                        ans.setElement(i,i,f.map(m.getElement(i,i)));
                        ans.setElement(i,i+1,f.map(m.getElement(i+1,i)));
                }
                ans.setElement(mRow,mRow-1,f.map(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,f.map(m.getElement(mRow,mRow)));
                return ans;
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m an integer diagonal matrix
        * @param f a user-defined function
        * @return a double diagonal matrix
        */
        public static DoubleDiagonalMatrix mapElements(IntegerDiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                double array[]=new double[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=f.map(m.getElement(i,i));
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Applies a function on all the matrix elements.
        * @param m a double matrix
        * @param f a user-defined function
        * @return a double matrix
        */
        public static DoubleMatrix mapElements(DoubleMatrix m, Mapping f) {
                int mRow=m.rows(),mCol=m.columns();
                double array[][]=new double[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new DoubleMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a double square matrix
        * @param f a user-defined function
        * @return a double square matrix
        */
        public static DoubleSquareMatrix mapElements(DoubleSquareMatrix m, Mapping f) {
                int mRow=m.rows();
                double array[][]=new double[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new DoubleSquareMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a double tridiagonal matrix
        * @param f a user-defined function
        * @return a double tridiagonal matrix
        */
        public static DoubleTridiagonalMatrix mapElements(DoubleTridiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                DoubleTridiagonalMatrix ans=new DoubleTridiagonalMatrix(mRow);
                ans.setElement(0,0,f.map(m.getElement(0,0)));
                ans.setElement(0,1,f.map(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,f.map(m.getElement(i-1,i)));
                        ans.setElement(i,i,f.map(m.getElement(i,i)));
                        ans.setElement(i,i+1,f.map(m.getElement(i+1,i)));
                }
                ans.setElement(mRow,mRow-1,f.map(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,f.map(m.getElement(mRow,mRow)));
                return ans;
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a double diagonal matrix
        * @param f a user-defined function
        * @return a double diagonal matrix
        */
        public static DoubleDiagonalMatrix mapElements(DoubleDiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                double array[]=new double[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=f.map(m.getElement(i,i));
                return new DoubleDiagonalMatrix(array);
        }

        /**
        * Applies a function on all the matrix elements.
        * @param m a complex matrix
        * @param f a user-defined function
        * @return a complex matrix
        */
        public static ComplexMatrix mapElements(ComplexMatrix m, Mapping f) {
                int mRow=m.rows(),mCol=m.columns();
                Complex array[][]=new Complex[mRow][mCol];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mCol;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new ComplexMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a complex square matrix
        * @param f a user-defined function
        * @return a complex square matrix
        */
        public static ComplexSquareMatrix mapElements(ComplexSquareMatrix m, Mapping f) {
                int mRow=m.rows();
                Complex array[][]=new Complex[mRow][mRow];
                for(int j,i=0;i<mRow;i++) {
                        for(j=0;j<mRow;j++)
                                array[i][j]=f.map(m.getElement(i,j));
                }
                return new ComplexSquareMatrix(array);
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a complex tridiagonal matrix
        * @param f a user-defined function
        * @return a complex tridiagonal matrix
        */
        public static ComplexTridiagonalMatrix mapElements(ComplexTridiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                ComplexTridiagonalMatrix ans=new ComplexTridiagonalMatrix(mRow);
                ans.setElement(0,0,f.map(m.getElement(0,0)));
                ans.setElement(0,1,f.map(m.getElement(1,0)));
                mRow--;
                for(int i=1;i<mRow;i++) {
                        ans.setElement(i,i-1,f.map(m.getElement(i-1,i)));
                        ans.setElement(i,i,f.map(m.getElement(i,i)));
                        ans.setElement(i,i+1,f.map(m.getElement(i+1,i)));
                }
                ans.setElement(mRow,mRow-1,f.map(m.getElement(mRow-1,mRow)));
                ans.setElement(mRow,mRow,f.map(m.getElement(mRow,mRow)));
                return ans;
        }
        /**
        * Applies a function on all the matrix elements.
        * @param m a complex diagonal matrix
        * @param f a user-defined function
        * @return a complex diagonal matrix
        */
        public static ComplexDiagonalMatrix mapElements(ComplexDiagonalMatrix m, Mapping f) {
                int mRow=m.rows();
                Complex array[]=new Complex[mRow];
                for(int i=0;i<mRow;i++)
                        array[i]=f.map(m.getElement(i,i));
                return new ComplexDiagonalMatrix(array);
        }

// LU DECOMPOSITION

        /**
        * Returns the LU decomposition of a matrix.
        * @param m a double square matrix
        * @return an array with [0] containing the L-matrix and [1] containing the U-matrix.
        */
        public static DoubleSquareMatrix[] LUDecomposition(DoubleSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                double tmp,array[][][]=new double[2][mRow][mRow];
                for(i=0;i<mRow;i++)
                        array[0][i][i]=1.0;
                for(j=0;j<mRow;j++) {
                        for(i=0;i<=j;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<i;k++)
                                        tmp-=array[0][i][k]*array[1][k][j];
                                array[1][i][j]=tmp;
                        }
                        for(i=j+1;i<mRow;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<j;k++)
                                        tmp-=array[0][i][k]*array[1][k][j];
                                array[0][i][j]=tmp/array[1][j][j];
                        }
                }
                DoubleSquareMatrix LU[]=new DoubleSquareMatrix[2];
                LU[0]=new DoubleSquareMatrix(array[0]);
                LU[1]=new DoubleSquareMatrix(array[1]);
                return LU;
        }
        /**
        * Returns the LU decomposition of a matrix.
        * @param m a complex square matrix
        * @return an array with [0] containing the L-matrix and [1] containing the U-matrix.
        */
        public static ComplexSquareMatrix[] LUDecomposition(ComplexSquareMatrix m) {
                int i,j,k,mRow=m.rows();
                Complex tmp,array[][][]=new Complex[2][mRow][mRow];
                for(i=0;i<mRow;i++)
                        array[0][i][i]=Complex.ONE;
                for(j=0;j<mRow;j++) {
                        for(i=0;i<=j;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<i;k++)
                                        tmp=ComplexMath.subtract(tmp,ComplexMath.multiply(array[0][i][k],array[1][k][j]));
                                array[1][j][i]=Complex.ZERO;
                                array[1][i][j]=tmp;
                        }
                        for(i=j+1;i<mRow;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<j;k++)
                                        tmp=ComplexMath.subtract(tmp,ComplexMath.multiply(array[0][i][k],array[1][k][j]));
                                array[0][j][i]=Complex.ZERO;
                                array[0][i][j]=ComplexMath.divide(tmp,array[1][j][j]);
                        }
                }
                ComplexSquareMatrix LU[]=new ComplexSquareMatrix[2];
                LU[0]=new ComplexSquareMatrix(array[0]);
                LU[1]=new ComplexSquareMatrix(array[1]);
                return LU;
        }
        /**
        * Returns the LU decomposition of a matrix.
        * @param m an integer square matrix
        * @return an array with [0] containing the L-matrix and [1] containing the U-matrix.
        */
        public static DoubleSquareMatrix[] LUDecomposition(IntegerSquareMatrix m) {
                int i,j,k,mRow=m.rows(),tmp;
                double array[][][]=new double[2][mRow][mRow];
                for(i=0;i<mRow;i++)
                        array[0][i][i]=1.0;
                for(j=0;j<mRow;j++) {
                        for(i=0;i<=j;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<i;k++)
                                        tmp-=array[0][i][k]*array[1][k][j];
                                array[1][i][j]=tmp;
                        }
                        for(i=j+1;i<mRow;i++) {
                                tmp=m.getElement(i,j);
                                for(k=0;k<j;k++)
                                        tmp-=array[0][i][k]*array[1][k][j];
                                array[0][i][j]=tmp/array[1][j][j];
                        }
                }
                DoubleSquareMatrix LU[]=new DoubleSquareMatrix[2];
                LU[0]=new DoubleSquareMatrix(array[0]);
                LU[1]=new DoubleSquareMatrix(array[1]);
                return LU;
        }

// EIGENVALUES & EIGENVECTORS

        /**
        * This method finds the eigenvalues of a symmetric tridiagonal matrix by the QL method.
        * It is based on the NETLIB algol/fortran procedure tql1 by Bowdler, Martin, Reinsch and Wilkinson.
        * @param matrix a double symmetric tridiagonal matrix
        * @return an array containing the eigenvalues
        * @exception MaximumIterationsExceededException If it takes more than 50 iterations to determine an eigenvalue.
        */
        public static double[] eigenvalueSolveSymmetric(DoubleTridiagonalMatrix matrix) throws MaximumIterationsExceededException {
                int i,j,n=matrix.rows(),nm1=n-1;
                double eigenvalue[]=new double[n];
                double offdiag[]=new double[n];
                for(i=0;i<nm1;i++) {
                        eigenvalue[i]=matrix.getElement(i,i);
                        offdiag[i]=matrix.getElement(i,i+1);
                }
                eigenvalue[nm1]=matrix.getElement(nm1,nm1);
                offdiag[nm1]=0.0;
                eigenvalueSolveSymmetricTridiagonalMatrix(eigenvalue,offdiag);
                return eigenvalue;
        }
        /**
        * This method finds the eigenvalues and eigenvectors of a symmetric tridiagonal matrix by the QL method.
        * It is based on the NETLIB algol/fortran procedure tql2 by Bowdler, Martin, Reinsch and Wilkinson.
        * @param matrix a double symmetric tridiagonal matrix
        * @param eigenvector an empty array of double vectors to hold the eigenvectors
        * @return an array containing the eigenvalues
        * @exception MaximumIterationsExceededException If it takes more than 50 iterations to determine an eigenvalue.
        */
        public static double[] eigenSolveSymmetric(DoubleTridiagonalMatrix matrix,DoubleVector eigenvector[]) throws MaximumIterationsExceededException {
                int i,j,n=matrix.rows(),nm1=n-1;
                double eigenvalue[]=new double[n];
                double offdiag[]=new double[n];
                double id[][]=new double[n][n];
                for(i=0;i<nm1;i++) {
                        id[i][i]=1.0;
                        eigenvalue[i]=matrix.getElement(i,i);
                        offdiag[i]=matrix.getElement(i,i+1);
                }
                id[nm1][nm1]=1.0;
                eigenvalue[nm1]=matrix.getElement(nm1,nm1);
                offdiag[nm1]=0.0;
                eigenSolveSymmetricTridiagonalMatrix(eigenvalue,offdiag,id);
                for(i=0;i<n;i++) {
                        eigenvector[i]=new DoubleVector(n);
                        for(j=0;j<n;j++)
                                eigenvector[i].setComponent(j,id[j][i]);
                }
                return eigenvalue;
        }
        /**
        * This method finds the eigenvalues of a symmetric square matrix.
        * The matrix is reduced to tridiagonal form and then the QL method is applied.
        * It is based on the NETLIB algol/fortran procedure tred1/tql1 by Bowdler, Martin, Reinsch and Wilkinson.
        * @param matrix a double symmetric square matrix
        * @return an array containing the eigenvalues
        * @exception MaximumIterationsExceededException If it takes more than 50 iterations to determine an eigenvalue.
        */
        public static double[] eigenvalueSolveSymmetric(DoubleSquareMatrix matrix) throws MaximumIterationsExceededException {
                int i,j,n=matrix.rows();
                double eigenvalue[]=new double[n];
                double offdiag[]=new double[n];
                double array[][]=new double[n][n];
                for(i=0;i<n;i++) {
                        for(j=0;j<n;j++)
                                array[i][j]=matrix.getElement(i,j);
                }
                reduceSymmetric1_SquareToTridiagonal(array,eigenvalue,offdiag);
                for(i=1;i<n;i++)
                        offdiag[i-1]=offdiag[i];
                offdiag[n-1]=0.0;
                eigenvalueSolveSymmetricTridiagonalMatrix(eigenvalue,offdiag);
                return eigenvalue;
        }
        /**
        * This method finds the eigenvalues and eigenvectors of a symmetric square matrix.
        * The matrix is reduced to tridiagonal form and then the QL method is applied.
        * It is based on the NETLIB algol/fortran procedure tred2/tql2 by Bowdler, Martin, Reinsch and Wilkinson.
        * @param matrix a double symmetric square matrix
        * @param eigenvector an empty array of double vectors to hold the eigenvectors
        * @return an array containing the eigenvalues
        * @exception MaximumIterationsExceededException If it takes more than 50 iterations to determine an eigenvalue.
        */
        public static double[] eigenSolveSymmetric(DoubleSquareMatrix matrix,DoubleVector eigenvector[]) throws MaximumIterationsExceededException {
                int i,j,n=matrix.rows();
                double eigenvalue[]=new double[n];
                double offdiag[]=new double[n];
                double transf[][]=new double[n][n];
                for(i=0;i<n;i++) {
                        for(j=0;j<n;j++)
                                transf[i][j]=matrix.getElement(i,j);
                }
                reduceSymmetric2_SquareToTridiagonal(transf,eigenvalue,offdiag);
                for(i=1;i<n;i++)
                        offdiag[i-1]=offdiag[i];
                offdiag[n-1]=0.0;
                eigenSolveSymmetricTridiagonalMatrix(eigenvalue,offdiag,transf);
                for(i=0;i<n;i++) {
                        eigenvector[i]=new DoubleVector(n);
                        for(j=0;j<n;j++)
                                eigenvector[i].setComponent(j,transf[j][i]);
                }
                return eigenvalue;
        }


        /**
        * Internal NETLIB tql1 routine.
        * @param diag output eigenvalues
        * @author Richard Cannings
        */
        private native static void eigenvalueSolveSymmetricTridiagonalMatrix(double diag[],double offdiag[]) throws MaximumIterationsExceededException;
        /**
        * Internal NETLIB tred1 routine.
        * @author Richard Cannings
        */
        private native static void reduceSymmetric1_SquareToTridiagonal(double matrix[][],double diag[],double offdiag[]);
        /**
        * Internal NETLIB tql2 routine.
        * @param diag output eigenvalues
        * @param transf output eigenvectors
        * @author Richard Cannings
        */
        private native static void eigenSolveSymmetricTridiagonalMatrix(double diag[],double offdiag[],double transf[][]) throws MaximumIterationsExceededException;
        /**
        * Internal NETLIB tred2 routine.
        * @param matrix output orthogonal transformations
        * @author Richard Cannings
        */
        private native static void reduceSymmetric2_SquareToTridiagonal(double matrix[][],double diag[],double offdiag[]);
}

