package JSci.awt;

import java.awt.*;
import JSci.maths.*;

/**
* An Argand diagram AWT component.
* @version 0.5
* @author Mark Hale
*/
public final class ArgandDiagram extends Canvas {
        /**
        * Data points.
        */
        private Complex data[];
        /**
        * Min and max data points.
        */
        private double minX,minY,maxX,maxY;
        /**
        * Origin.
        */
        private Point origin;
        /**
        * Axis scaling.
        */
        private double xScale,yScale;
        /**
        * Line colors.
        */
        private Color lineColor=Color.black;
        /**
        * Axis numbering.
        */
        private boolean numbering=true;
        /**
        * Padding.
        */
        private final int scalePad=5,axisPad=25;
        /**
        * Construct an Argand diagram.
        * @param array an array of complex points
        */
        public ArgandDiagram(Complex array[]) {
                setData(array);
        }
        /**
        * Sets the data plotted by this ArgandDiagram to the specified data.
        * @param array an array of complex points
        */
        public void setData(Complex array[]) {
                double re,im;
                data=new Complex[array.length];
                minX=maxX=minY=maxY=0.0;
                for(int i=0;i<data.length;i++) {
                        data[i]=array[i];
                        re=data[i].real();
                        if(re>maxX)
                                maxX=re;
                        else if(re<minX)
                                minX=re;
                        im=data[i].imag();
                        if(im>maxY)
                                maxY=im;
                        else if(im<minY)
                                minY=im;
                }
                rescale();
        }
        /**
        * Sets the line color.
        * @param c the line color
        */
        public void setColor(Color c) {
                lineColor=c;
        }
        /**
        * Turns axis numbering on/off.
        */
        public void setNumbering(boolean flag) {
                numbering=flag;
        }
        /**
        * Returns the preferred size of this component.
        */
        public Dimension preferredSize() {
                return minimumSize();
        }
        /**
        * Returns the minimum size of this component.
        */
        public Dimension minimumSize() {
                return new Dimension(250,250);
        }
        /**
        * Reshapes the ArgandDiagram to the specified bounding box.
        */
        public void reshape(int x,int y,int width,int height) {
                super.reshape(x,y,width,height);
                rescale();
        }
        /**
        * Rescales the ArgandDiagram.
        */
        private void rescale() {
                xScale=(size().width-2*(axisPad+scalePad))/(maxX-minX);
                yScale=(size().height-2*(axisPad+scalePad))/(maxY-minY);
                origin=new Point((int)Math.round(-minX*xScale)+axisPad+scalePad,(int)Math.round(maxY*yScale)+axisPad+scalePad);
                repaint();
        }
        /**
        * Converts a data point to screen coordinates.
        */
        private Point dataToScreen(double x,double y) {
                return new Point(origin.x+(int)Math.round(xScale*x),origin.y-(int)Math.round(yScale*y));
        }
        /**
        * Paint the diagram.
        */
        public void paint(Graphics g) {
// axis
                g.setColor(Color.black);
                g.drawLine(axisPad,origin.y,size().width-axisPad,origin.y);
                g.drawLine(origin.x,axisPad,origin.x,size().height-axisPad);
// numbering
                if(numbering) {
                        int numPad=g.getFontMetrics().getHeight();
                        int len;
// real-axis numbering
                        double x,dx=40.0/xScale;
                        String xStr,dxStr=String.valueOf(dx);
                        if(dx<1.0) {
                                double sdx=10*dx;
                                for(len=1;sdx<1.0;len++)
                                        sdx*=10;
                                len+=3;     // 2 + decimal point
                        } else
                                len=2;
                        if(dxStr.length()>len)
                                dx=Double.valueOf(dxStr.substring(0,len)).doubleValue();
                        for(x=dx;x<maxX;x+=dx) {
                                xStr=String.valueOf(x);
                                if(xStr.length()>len)
                                        xStr=xStr.substring(0,len);
                                g.drawString(xStr,dataToScreen(x,0.0).x,origin.y+numPad);
                        }
                        len++;  // minus sign
                        for(x=-dx;x>minX;x-=dx) {
                                xStr=String.valueOf(x);
                                if(xStr.length()>len)
                                        xStr=xStr.substring(0,len);
                                g.drawString(xStr,dataToScreen(x,0.0).x,origin.y+numPad);
                        }
// imag-axis numbering
                        double y,dy=40.0/yScale;
                        String yStr,dyStr=String.valueOf(dy);
                        if(dy<1.0) {
                                double sdy=10*dy;
                                for(len=1;sdy<1.0;len++)
                                        sdy*=10;
                                len+=3;     // 2 + decimal point
                        } else
                                len=2;
                        if(dyStr.length()>len)
                                dy=Double.valueOf(dyStr.substring(0,len)).doubleValue();
                        for(y=dy;y<maxY;y+=dy) {
                                yStr=String.valueOf(y);
                                if(yStr.length()>len)
                                        yStr=yStr.substring(0,len);
                                numPad=g.getFontMetrics().stringWidth(yStr)+5;
                                g.drawString(yStr,origin.x-numPad,dataToScreen(0.0,y).y);
                        }
                        len++;  // minus sign
                        for(y=-dy;y>minY;y-=dy) {
                                yStr=String.valueOf(y);
                                if(yStr.length()>len)
                                        yStr=yStr.substring(0,len);
                                numPad=g.getFontMetrics().stringWidth(yStr)+5;
                                g.drawString(yStr,origin.x-numPad,dataToScreen(0.0,y).y);
                        }
                }
// lines
                g.setColor(lineColor);
                Point p1,p2;
                p1=dataToScreen(data[0].real(),data[0].imag());
                int i;
                for(i=1;i<data.length;i++) {
                        p2=dataToScreen(data[i].real(),data[i].imag());
                        g.drawLine(p1.x,p1.y,p2.x,p2.y);
                        p1=p2;
                }
        }
}

