package JSci.awt;

import java.awt.*;

/**
* A line graph AWT component.
* @version 0.5
* @author Mark Hale
*/
public final class LineGraph extends Canvas {
        /**
        * Data points.
        */
        private double data[][];
        /**
        * Min and max data points.
        */
        private double minX,minY,maxX,maxY;
        /**
        * Origin.
        */
        private Point origin;
        /**
        * Axis scaling.
        */
        private double xScale,yScale;
        /**
        * Line colors.
        */
        private Color lineColor[]={Color.black,Color.blue,Color.green,Color.red,Color.yellow};
        /**
        * Axis numbering.
        */
        private boolean numbering=true;
        /**
        * Padding.
        */
        private final int scalePad=5,axisPad=25;
        /**
        * Construct a line graph.
        * @param array [0][] contains the x-series, [n][] contains the nth y-series
        */
        public LineGraph(double array[][]) {
                setData(array);
        }
        /**
        * Sets the data plotted by this LineGraph to the specified data.
        * @param array [0][] contains the x-series, [n][] contains the nth y-series
        */
        public void setData(double array[][]) {
                int i;
                data=new double[array.length][array[0].length];
                minX=maxX=minY=maxY=0.0;
                if(data.length>lineColor.length) {
                        Color tmp[]=lineColor;
                        lineColor=new Color[data.length-1];
                        for(i=0;i<tmp.length;i++)
                                lineColor[i]=tmp[i];
                        for(i=tmp.length;i<lineColor.length;i++)
                                lineColor[i]=Color.black;
                }
                for(i=0;i<data[0].length;i++) {
                        data[0][i]=array[0][i];
                        if(data[0][i]>maxX)
                                maxX=data[0][i];
                        else if(data[0][i]<minX)
                                minX=data[0][i];
                        data[1][i]=array[1][i];
                        if(data[1][i]>maxY)
                                maxY=data[1][i];
                        else if(data[1][i]<minY)
                                minY=data[1][i];
                }
                for(int n=2;n<data.length;n++) {
                        for(i=0;i<data[0].length;i++) {
                                data[n][i]=array[n][i];
                                if(data[n][i]>maxY)
                                        maxY=data[n][i];
                                else if(data[n][i]<minY)
                                        minY=data[n][i];
                        }
                }
                rescale();
        }
        /**
        * Sets the line color of the nth y-series.
        * @param n the index of the y-series
        * @param c the line color
        */
        public void setColor(int n,Color c) {
                lineColor[n]=c;
        }
        /**
        * Turns axis numbering on/off.
        */
        public void setNumbering(boolean flag) {
                numbering=flag;
        }
        /**
        * Returns the preferred size of this component.
        */
        public Dimension preferredSize() {
                return minimumSize();
        }
        /**
        * Returns the minimum size of this component.
        */
        public Dimension minimumSize() {
                return new Dimension(250,250);
        }
        /**
        * Reshapes the LineGraph to the specified bounding box.
        */
        public void reshape(int x,int y,int width,int height) {
                super.reshape(x,y,width,height);
                rescale();
        }
        /**
        * Rescales the LineGraph.
        */
        private void rescale() {
                xScale=(size().width-2*(axisPad+scalePad))/(maxX-minX);
                yScale=(size().height-2*(axisPad+scalePad))/(maxY-minY);
                origin=new Point((int)Math.round(-minX*xScale)+axisPad+scalePad,(int)Math.round(maxY*yScale)+axisPad+scalePad);
                repaint();
        }
        /**
        * Converts a data point to screen coordinates.
        */
        private Point dataToScreen(double x,double y) {
                return new Point(origin.x+(int)Math.round(xScale*x),origin.y-(int)Math.round(yScale*y));
        }
        /**
        * Paint the graph.
        */
        public void paint(Graphics g) {
// axis
                g.setColor(Color.black);
                g.drawLine(axisPad,origin.y,size().width-axisPad,origin.y);
                g.drawLine(origin.x,axisPad,origin.x,size().height-axisPad);
// numbering
                if(numbering) {
                        int numPad=g.getFontMetrics().getHeight();
                        int len;
// x-axis numbering
                        double x,dx=40.0/xScale;
                        String xStr,dxStr=String.valueOf(dx);
                        if(dx<1.0) {
                                double sdx=10*dx;
                                for(len=1;sdx<1.0;len++)
                                        sdx*=10;
                                len+=3;     // 2 + decimal point
                        } else
                                len=2;
                        if(dxStr.length()>len)
                                dx=Double.valueOf(dxStr.substring(0,len)).doubleValue();
                        for(x=dx;x<maxX;x+=dx) {
                                xStr=String.valueOf(x);
                                if(xStr.length()>len)
                                        xStr=xStr.substring(0,len);
                                g.drawString(xStr,dataToScreen(x,0.0).x,origin.y+numPad);
                        }
                        len++;  // minus sign
                        for(x=-dx;x>minX;x-=dx) {
                                xStr=String.valueOf(x);
                                if(xStr.length()>len)
                                        xStr=xStr.substring(0,len);
                                g.drawString(xStr,dataToScreen(x,0.0).x,origin.y+numPad);
                        }
// y-axis numbering
                        double y,dy=40.0/yScale;
                        String yStr,dyStr=String.valueOf(dy);
                        if(dy<1.0) {
                                double sdy=10*dy;
                                for(len=1;sdy<1.0;len++)
                                        sdy*=10;
                                len+=3;     // 2 + decimal point
                        } else
                                len=2;
                        if(dyStr.length()>len)
                                dy=Double.valueOf(dyStr.substring(0,len)).doubleValue();
                        for(y=dy;y<maxY;y+=dy) {
                                yStr=String.valueOf(y);
                                if(yStr.length()>len)
                                        yStr=yStr.substring(0,len);
                                numPad=g.getFontMetrics().stringWidth(yStr)+5;
                                g.drawString(yStr,origin.x-numPad,dataToScreen(0.0,y).y);
                        }
                        len++;  // minus sign
                        for(y=-dy;y>minY;y-=dy) {
                                yStr=String.valueOf(y);
                                if(yStr.length()>len)
                                        yStr=yStr.substring(0,len);
                                numPad=g.getFontMetrics().stringWidth(yStr)+5;
                                g.drawString(yStr,origin.x-numPad,dataToScreen(0.0,y).y);
                        }
                }
// lines
                g.setColor(lineColor[0]);
                Point p1,p2;
                p1=dataToScreen(data[0][0],data[1][0]);
                int i;
                for(i=1;i<data[0].length;i++) {
                        p2=dataToScreen(data[0][i],data[1][i]);
                        g.drawLine(p1.x,p1.y,p2.x,p2.y);
                        p1=p2;
                }
                for(int n=2;n<data.length;n++) {
                        g.setColor(lineColor[n-1]);
                        p1=dataToScreen(data[0][0],data[n][0]);
                        for(i=1;i<data[0].length;i++) {
                                p2=dataToScreen(data[0][i],data[n][i]);
                                g.drawLine(p1.x,p1.y,p2.x,p2.y);
                                p1=p2;
                        }
                }
        }
}

