package JSci.maths;

/**
* The ComplexDiagonalMatrix class provides an object for encapsulating diagonal matrices containing complex numbers.
* @version 1.6
* @author Mark Hale
*/
public final class ComplexDiagonalMatrix extends ComplexTridiagonalMatrix {
        /**
        * Constructs a matrix.
        */
        private ComplexDiagonalMatrix() {
                super();
        }
        /**
        * Constructs an empty matrix.
        * @param size the number of rows/columns
        */
        public ComplexDiagonalMatrix(int size) {
                this();
                matrix=new Complex[1][size];
        }
        /**
        * Constructs a matrix from an array.
        * @param array an assigned value
        * @exception MatrixDimensionException If the array is not square.
        */
        public ComplexDiagonalMatrix(Complex array[][]) {
                this(array.length);
                if(array.length==array[0].length) {
                        for(int i=0;i<matrix[0].length;i++)
                                matrix[0][i]=array[i][i];
                } else {
                        matrix=null;
                        throw new MatrixDimensionException("Array must be square.");
                }
        }
        /**
        * Constructs a matrix by wrapping an array containing the diagonal elements.
        * @param array an assigned value
        */
        public ComplexDiagonalMatrix(Complex array[]) {
                this();
                matrix=new Complex[1][];
                matrix[0]=array;
        }
        /**
        * Creates an identity matrix.
        * @param size the number of rows/columns
        */
        public static ComplexDiagonalMatrix identity(int size) {
                Complex array[]=new Complex[size];
                for(int i=0;i<size;i++)
                        array[i]=Complex.ONE;
                return new ComplexDiagonalMatrix(array);
        }
        /**
        * Compares two complex diagonal matrices for equality.
        * @param m a complex diagonal matrix
        */
        public boolean equals(Object m) {
                if(m!=null && (m instanceof ComplexDiagonalMatrix) &&
                matrix[0].length==((ComplexDiagonalMatrix)m).rows()) {
                        ComplexDiagonalMatrix cdm=(ComplexDiagonalMatrix)m;
                        for(int i=0;i<matrix[0].length;i++) {
                                if(matrix[0][i]!=cdm.getElement(i,i))
                                        return false;
                        }
                        return true;
                } else
                        return false;
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                Complex array[]=new Complex[matrix[0].length];
                for(int i=0;i<matrix[0].length;i++)
                        array[i]=(Complex)matrix[0][i].clone();
                return new ComplexDiagonalMatrix(array);
        }
        /**
        * Returns the real part of this complex matrix.
        * @return a double diagonal matrix
        */
        public DoubleMatrix real() {
                double array[]=new double[matrix[0].length];
                for(int i=0;i<matrix[0].length;i++)
                        array[i]=matrix[0][i].real();
                return new DoubleDiagonalMatrix(array);
        }
        /**
        * Returns the imaginary part of this complex matrix.
        * @return a double diagonal matrix
        */
        public DoubleMatrix imag() {
                double array[]=new double[matrix[0].length];
                for(int i=0;i<matrix[0].length;i++)
                        array[i]=matrix[0][i].imag();
                return new DoubleDiagonalMatrix(array);
        }
        /**
        * Returns an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public Complex getElement(int i, int j) {
                if(i>=0 && i<matrix[0].length && j>=0 && j<matrix[0].length) {
                        if(i==j)
                                return matrix[0][i];
                        else
                                return Complex.ZERO;
                } else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Sets the value of an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @param z a complex number
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public void setElement(int i, int j, Complex z) {
                if(i>=0 && i<matrix[0].length && j>=0 && j<matrix[0].length && i==j)
                        matrix[0][i]=z;
                else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Returns true if this matrix is hermitian.
        */
        public boolean isHermitian() {
                return equals(MatrixMath.conjugate(this));
        }
        /**
        * Returns true if this matrix is unitary.
        */
        public boolean isUnitary() {
                return identity(matrix[0].length).equals(MatrixMath.multiply(this,MatrixMath.conjugate(this)));
        }
        /**
        * Returns the determinant.
        */
        public Complex determinant() {
                Complex det=matrix[0][0];
                for(int i=1;i<matrix[0].length;i++)
                        det=ComplexMath.multiply(det,matrix[0][i]);
                return det;
        }
        /**
        * Returns the trace.
        */
        public Complex trace() {
                Complex tr=matrix[0][0];
                for(int i=1;i<matrix[0].length;i++)
                        tr=ComplexMath.add(tr,matrix[0][i]);
                return tr;
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                double result=0.0, tmpResult;
                for(int i=0;i<matrix[0].length;i++) {
                        tmpResult=matrix[0][i].mod();
                        if (tmpResult >= result)
                                result = tmpResult;
                }
                return result;
        }
        /**
        * Returns the Frobenius norm.
        * @author Taber Smith
        */
        public double frobeniusNorm() {
                double result=matrix[0][0].mod()*matrix[0][0].mod();
                for(int i=1;i<matrix[0].length;i++)
                        result+=matrix[0][i].mod()*matrix[0][i].mod();
                return Math.sqrt(result);
        }
        /**
        * Returns the number of rows.
        */
        public int rows() {
                return matrix[0].length;
        }
        /**
        * Returns the number of columns.
        */
        public int columns() {
                return matrix[0].length;
        }
}

