package JSci.maths;

/**
* The complex math library.
* This class cannot be subclassed or instantiated because all methods are static.
* @version 1.6
* @author Mark Hale
*/
public final class ComplexMath extends Object {
        private ComplexMath() {}

        /**
        * Returns the negative of a complex number.
        * @param z a complex number
        */
        public static Complex negate(Complex z) {
                return new Complex(-z.real(), -z.imag());
        }
        /**
        * Returns the complex conjugate of a complex number.
        * @param z a complex number
        */
        public static Complex conjugate(Complex z) {
                return new Complex(z.real(), -z.imag());
        }
        /**
        * Returns the sum of the complex numbers in an array.
        * @param zarray an array of complex numbers
        */
        public static Complex sum(Complex zarray[]) {
                Complex answer=zarray[0];
                for(int i=1;i<zarray.length;i++)
                        answer=add(answer,zarray[i]);
                return answer;
        }
        /**
        * Returns the product of the complex numbers in an array.
        * @param zarray an array of complex numbers
        */
        public static Complex product(Complex zarray[]) {
                Complex answer=zarray[0];
                for(int i=1;i<zarray.length;i++)
                        answer=multiply(answer,zarray[i]);
                return answer;
        }

// ADDITION

        /**
        * Returns the addition of two complex numbers (z1+z2).
        * @param z1 a complex number
        * @param z2 a complex number
        */
        public static Complex add(Complex z1, Complex z2) {
                return new Complex(z1.real()+z2.real(), z1.imag()+z2.imag());
        }
        /**
        * Returns the addition of a complex number and a real part.
        * @param z a complex number
        * @param re a real part
        */
        public static Complex addReal(Complex z, double re) {
                return new Complex(z.real()+re, z.imag());
        }
        /**
        * Returns the addition of a complex number and an imaginary part.
        * @param z a complex number
        * @param im an imaginary part
        */
        public static Complex addImag(Complex z, double im) {
                return new Complex(z.real(), z.imag()+im);
        }
        /**
        * Returns the addition of three complex numbers (z1+z2+z3).
        * @param z1 a complex number
        * @param z2 a complex number
        * @param z3 a complex number
        */
        public static Complex add(Complex z1, Complex z2, Complex z3) {
                return new Complex(z1.real()+z2.real()+z3.real(), z1.imag()+z2.imag()+z3.imag());
        }

// SUBTRACTION

        /**
        * Returns the subtraction of two complex numbers (z1-z2).
        * @param z1 a complex number
        * @param z2 a complex number
        */
        public static Complex subtract(Complex z1, Complex z2) {
                return new Complex(z1.real()-z2.real(), z1.imag()-z2.imag());
        }
        /**
        * Returns the subtraction of a complex number by a real part.
        * @param z a complex number
        * @param re a real part
        */
        public static Complex subtractReal(Complex z, double re) {
                return new Complex(z.real()-re, z.imag());
        }
        /**
        * Returns the subtraction of a complex number by an imaginary part.
        * @param z a complex number
        * @param im an imaginary part
        */
        public static Complex subtractImag(Complex z, double im) {
                return new Complex(z.real(), z.imag()-im);
        }

// MULTIPLICATION

        /**
        * Returns the multiplication of two complex numbers (z1*z2).
        * @param z1 a complex number
        * @param z2 a complex number
        */
        public static Complex multiply(Complex z1, Complex z2) {
                double z1Re=z1.real(),z1Im=z1.imag();
                double z2Re=z2.real(),z2Im=z2.imag();
                return new Complex(
                        z1Re*z2Re-z1Im*z2Im ,
                        z1Re*z2Im+z1Im*z2Re
                );
        }
        /**
        * Returns the multiplication of three complex numbers (z1*z2*z3).
        * @param z1 a complex number
        * @param z2 a complex number
        * @param z3 a complex number
        */
        public static Complex multiply(Complex z1, Complex z2, Complex z3) {
                double a=z1.real()*z2.real()-z1.imag()*z2.imag();
                double b=z1.real()*z2.imag()+z2.real()*z1.imag();
                return new Complex(
                        z3.real()*a-z3.imag()*b ,
                        z3.real()*b+z3.imag()*a
                );
        }
        /**
        * Returns the multiplication of a complex number by a scalar.
        * @param x a real number
        * @param z a complex number
        */
        public static Complex multiply(double x, Complex z) {
                return new Complex(x*z.real(), x*z.imag());
        }

// DIVISION

        /**
        * Returns the division of two complex numbers (z1/z2).
        * @param z1 a complex number
        * @param z2 a complex number
        */
        public static Complex divide(Complex z1, Complex z2) {
                double z1Re=z1.real(),z1Im=z1.imag();
                double z2Re=z2.real(),z2Im=z2.imag();
                double denominator,re,im,a;             
                if(Math.abs(z2Re)<Math.abs(z2Im)) {
                        a=z2Re/z2Im;
                        denominator=z2Re*a+z2Im;
                        re=z1Re*a+z1Im;
                        im=z1Im*a-z1Re;
                } else {
                        a=z2Im/z2Re;
                        denominator=z2Re+z2Im*a;
                        re=z1Re+z1Im*a;
                        im=z1Im-z1Re*a;
                }
                return new Complex(re/denominator, im/denominator);
        }
        /**
        * Returns the division of a complex number by a scalar.
        * @param z a complex number
        * @param x a real number
        */
        public static Complex divide(Complex z, double x) {
                return new Complex(z.real()/x, z.imag()/x);
        }

// POWER

        /**
        * Returns the complex number z1 raised to the power of z2.
        * @param z1 a complex number
        * @param z2 a complex number
        */
        public static Complex pow(Complex z1, Complex z2) {
                double z1Mod=z1.mod(),z1Arg=z1.arg();
                double z2Re=z2.real(),z2Im=z2.imag();
                double r=Math.pow(z1Mod, z2Re)/Math.exp(z1Arg*z2Im);
                double a=z1Arg*z2Re+Math.log(z1Mod)*z2Im;
                return Complex.polar(r,a);
        }
        /**
        * Returns the complex number z raised to the power of x.
        * @param z a complex number
        * @param x a real number
        */
        public static Complex pow(Complex z, double x) {
                return Complex.polar(Math.pow(z.mod(), x), z.arg()*x);
        }
        /**
        * Returns the square of a complex number.
        * @param z a complex number
        */
        public static Complex sqr(Complex z) {
                double zRe=z.real(),zIm=z.imag();
                return new Complex(zRe*zRe-zIm*zIm ,zRe*zIm+zIm*zRe);
        }
        /**
        * Returns the square root of a complex number.
        * @param z a complex number
        */
        public static Complex sqrt(Complex z) {
                return Complex.polar(Math.pow(z.mod(), 0.5), z.arg()*0.5);
        }

// EXP

        /**
        * Returns the exponential number e(2.718...) raised to the power of z.
        * @param z a complex number
        */
        public static Complex exp(Complex z) {
                double zRe=z.real(),zIm=z.imag();
                return new Complex(
                        Math.exp(zRe)*Math.cos(zIm) ,
                        Math.exp(zRe)*Math.sin(zIm)
                );
        }

// LOG

        /**
        * Returns the natural logarithm (base e) of z.
        * @param z a complex number
        */
        public static Complex log(Complex z) {
                return new Complex(Math.log(z.mod()), z.arg());
        }

// SIN

        /**
        * Returns the trigonometric sine of a complex angle.
        * @param z an angle that is measured in radians
        */
        public static Complex sin(Complex z) {
                double zRe=z.real(),zIm=z.imag();
                return new Complex(
                        Math.sin(zRe)*cosh(zIm) ,
                        Math.cos(zRe)*sinh(zIm)
                );
        }

// COS

        /**
        * Returns the trigonometric cosine of a complex angle.
        * @param z an angle that is measured in radians
        */
        public static Complex cos(Complex z) {
                double zRe=z.real(),zIm=z.imag();
                return new Complex(
                        Math.cos(zRe)*cosh(zIm) ,
                       -Math.sin(zRe)*sinh(zIm)
                );
        }

// TAN

        /**
        * Returns the trigonometric tangent of a complex angle.
        * @param z an angle that is measured in radians
        */
        public static Complex tan(Complex z) {
                return divide(sin(z), cos(z));
        }

// SINH

        /**
        * Returns the hyperbolic sine of a complex number.
        * @param z a complex number
        */
        public static Complex sinh(Complex z) {
                double zRe=z.real(),zIm=z.imag();
                return new Complex(
                        sinh(zRe)*Math.cos(zIm) ,
                        cosh(zRe)*Math.sin(zIm)
                );
        }

// COSH

        /**
        * Returns the hyperbolic cosine of a complex number.
        * @param z a complex number
        */
        public static Complex cosh(Complex z) {
                double zRe=z.real(),zIm=z.imag();
                return new Complex(
                        cosh(zRe)*Math.cos(zIm) ,
                        sinh(zRe)*Math.sin(zIm)
                );
        }

// TANH

        /**
        * Returns the hyperbolic tangent of a complex number.
        * @param z a complex number
        */
        public static Complex tanh(Complex z) {
                return divide(sinh(z), cosh(z));
        }

// COMPLEX CONSTANTS
        private final static Complex PI=new Complex(Math.PI,0.0);
        private final static Complex PI_I=new Complex(0.0,Math.PI);
        private final static Complex PI_2=new Complex(Math.PI/2.0,0.0);
        private final static Complex MINUS_PI_2=new Complex(-Math.PI/2.0,0.0);
        private final static Complex PI_2_I=new Complex(0.0,Math.PI/2.0);
        private final static Complex MINUS_PI_2_I=new Complex(0.0,-Math.PI/2.0);

// INVERSE SIN

        /**
        * Returns the arc sine of a complex number.
        * @param z a complex number
        */
        public static Complex asin(Complex z) {
                if(z.equals(Complex.ONE))
                        return PI_2;
                else if(z.equals(Complex.MINUS_ONE))
                        return MINUS_PI_2;
                else
                        return atan(divide(z,sqrt(subtract(Complex.ONE,sqr(z)))));
        }

// INVERSE COS

        /**
        * Returns the arc cosine of a complex number.
        * @param z a complex number
        */
        public static Complex acos(Complex z) {
                if(z.equals(Complex.ONE))
                        return Complex.ZERO;
                else if(z.equals(Complex.MINUS_ONE))
                        return PI;
                else
                        return addReal(atan(divide(negate(z),sqrt(subtract(Complex.ONE,sqr(z))))),Math.PI/2);
        }

// INVERSE TAN

        /**
        * Returns the arc tangent of a complex number.
        * @param z a complex number
        */
        public static Complex atan(Complex z) {
                return multiply(Complex.MINUS_I,atanh(multiply(Complex.I,z)));
        }

// ASINH

        /**
        * Returns the arc hyperbolic sine of a complex number.
        * @param z a complex number
        */
        public static Complex asinh(Complex z) {
                if(z.equals(Complex.I))
                        return PI_2_I;
                else if(z.equals(Complex.MINUS_I))
                        return MINUS_PI_2_I;
                else
                        return log(add(z,sqrt(addReal(sqr(z),1.0))));
        }

// ACOSH

        /**
        * Returns the arc hyperbolic cosine of a complex number.
        * @param z a complex number
        */
        public static Complex acosh(Complex z) {
                if(z.equals(Complex.ONE))
                        return Complex.ZERO;
                else if(z.equals(Complex.MINUS_ONE))
                        return PI_I;
                else
                        return log(add(z,sqrt(subtractReal(sqr(z),1.0))));
        }

// ATANH

        /**
        * Returns the arc hyperbolic tangent of a complex number.
        * @param z a complex number
        */
        public static Complex atanh(Complex z) {
                Complex omz=negate(subtractReal(z,1.0));
                Complex opz=addReal(z,1.0);
                return divide(log(divide(opz,omz)),2.0);
        }


        /**
        * Hyperbolic sine function.
        */
        private static double sinh(double x) {
                return (Math.exp(x)-Math.exp(-x))/2.0;
        }
        /**
        * Hyperbolic cosine function.
        */
        private static double cosh(double x) {
                return (Math.exp(x)+Math.exp(-x))/2.0;
        }
}

