package JSci.maths;

/**
* The ComplexMatrix class provides an object for encapsulating matrices containing complex numbers.
* @version 1.5
* @author Mark Hale
*/
public class ComplexMatrix extends Matrix {
        /**
        * Array containing the elements of the matrix.
        */
        protected Complex matrix[][];
        /**
        * Constructs a matrix.
        */
        protected ComplexMatrix() {}
        /**
        * Constructs an empty matrix.
        * @param row the number of rows
        * @param col the number of columns
        */
        public ComplexMatrix(int row,int col) {
                matrix=new Complex[row][col];
        }
        /**
        * Constructs a matrix by wrapping an array.
        * @param array an assigned value
        */
        public ComplexMatrix(Complex array[][]) {
                matrix=array;
        }
        /**
        * Constructs a matrix from an array of vectors.
        * @param array an assigned value
        */
        public ComplexMatrix(ComplexVector array[]) {
                this(array.length,array[0].dimension());
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                matrix[i][j]=array[i].getComponent(j);
                }
        }
        /**
        * Compares two complex matrices for equality.
        * @param m a complex matrix
        */
        public boolean equals(Object m) {
                if(m!=null && (m instanceof ComplexMatrix) &&
                matrix.length==((ComplexMatrix)m).rows() && matrix[0].length==((ComplexMatrix)m).columns()) {
                        ComplexMatrix cm=(ComplexMatrix)m;
                        for(int j,i=0;i<matrix.length;i++) {
                                for(j=0;j<matrix[0].length;j++) {
                                        if(!matrix[i][j].equals(cm.getElement(i,j))) {
                                                return false;
                                        }
                                }
                        }
                        return true;
                } else {
                        return false;
                }
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                Complex array[][]=new Complex[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                array[i][j]=(Complex)matrix[i][j].clone();
                }
                return new ComplexMatrix(array);
        }
        /**
        * Returns a string representing this matrix.
        */
        public String toString() {
                StringBuffer buf=new StringBuffer(matrix.length*matrix[0].length);
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++) {
                                buf.append(matrix[i][j].toString());
                                buf.append(' ');
                        }
                        buf.append('\n');
                }
                return buf.toString();
        }
        /**
        * Returns a hashcode for this matrix.
        */
        public int hashCode() {
                return (int)Math.exp(infNorm());
        }
        /**
        * Returns true if this matrix is hermitian.
        */
        public boolean isHermitian() {
                return equals(MatrixMath.hermitianAdjoint(this));
        }
        /**
        * Returns the real part of this complex matrix.
        * @return a double matrix
        */
        public DoubleMatrix real() {
                double array[][]=new double[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                array[i][j]=matrix[i][j].real();
                }
                return new DoubleMatrix(array);
        }
        /**
        * Returns the imaginary part of this complex matrix.
        * @return a double matrix
        */
        public DoubleMatrix imag() {
                double array[][]=new double[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                array[i][j]=matrix[i][j].imag();
                }
                return new DoubleMatrix(array);
        }
        /**
        * Returns an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public Complex getElement(int i, int j) {
                if(i>=0 && i<matrix.length && j>=0 && j<matrix[0].length)
                        return matrix[i][j];
                else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Sets the value of an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @param z a complex number
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public void setElement(int i, int j, Complex z) {
                if(i>=0 && i<matrix.length && j>=0 && j<matrix[0].length)
                        matrix[i][j]=z;
                else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                double result=0.0, tmpResult;
                for(int j,i=0;i<matrix.length;i++) {
                        tmpResult = 0.0;
                        for(j=0;j<matrix[0].length;j++)
                                tmpResult += matrix[i][j].mod();
                        if (tmpResult >= result)
                                result = tmpResult;
                }
                return result;
        }
        /**
        * Returns the Frobenius norm.
        * @author Taber Smith
        */
        public double frobeniusNorm() {
                double result=0.0;
                for(int j,i=0;i<matrix.length;i++)
                        for(j=0;j<matrix[0].length;j++)
                                result += matrix[i][j].mod()*matrix[i][j].mod();
                return Math.sqrt(result);
        }
        /**
        * Returns the number of rows.
        */
        public int rows() {
                return matrix.length;
        }
        /**
        * Returns the number of columns.
        */
        public int columns() {
                return matrix[0].length;
        }
}

