package JSci.maths;

/**
* The ComplexTridiagonalMatrix class provides an object for encapsulating tridiagonal matrices containing complex numbers.
* @version 1.6
* @author Mark Hale
*/
public class ComplexTridiagonalMatrix extends ComplexSquareMatrix {
        /**
        * Constructs a matrix.
        */
        protected ComplexTridiagonalMatrix() {
                super();
        }
        /**
        * Constructs an empty matrix.
        * @param size the number of rows/columns
        */
        public ComplexTridiagonalMatrix(int size) {
                this();
                matrix=new Complex[3][size];
        }
        /**
        * Constructs a matrix from an array.
        * @param array an assigned value
        * @exception MatrixDimensionException If the array is not square.
        */
        public ComplexTridiagonalMatrix(Complex array[][]) {
                this(array.length);
                if(array.length==array[0].length) {
                        matrix[1][0]=array[0][0];
                        matrix[2][0]=array[1][0];
                        for(int i=0;i<array.length-1;i++) {
                                matrix[0][i]=array[i-1][i];
                                matrix[1][i]=array[i][i];
                                matrix[2][i]=array[i+1][i];
                        }
                        matrix[0][array.length-1]=array[array.length-2][array.length-1];
                        matrix[1][array.length-1]=array[array.length-1][array.length-1];
                } else {
                        matrix=null;
                        throw new MatrixDimensionException("The array is not square.");
                }
        }
        /**
        * Compares two complex tridiagonal matrices for equality.
        * @param m a complex tridiagonal matrix
        */
        public boolean equals(Object m) {
                if(m!=null && (m instanceof ComplexTridiagonalMatrix) &&
                matrix[0].length==((ComplexTridiagonalMatrix)m).rows()) {
                        ComplexTridiagonalMatrix ctm=(ComplexTridiagonalMatrix)m;
                        if(matrix[1][0]!=ctm.getElement(0,0))
                                return false;
                        if(matrix[2][0]!=ctm.getElement(1,0))
                                return false;
                        for(int i=1;i<matrix[0].length-1;i++) {
                                if(matrix[0][i]!=ctm.getElement(i-1,i))
                                        return false;
                                if(matrix[1][i]!=ctm.getElement(i,i))
                                        return false;
                                if(matrix[2][i]!=ctm.getElement(i+1,i))
                                        return false;
                        }
                        if(matrix[0][matrix[0].length-1]!=ctm.getElement(matrix[0].length-2,matrix[0].length-1))
                                return false;
                        if(matrix[1][matrix[0].length-1]!=ctm.getElement(matrix[0].length-1,matrix[0].length-1))
                                return false;
                        return true;
                } else {
                        return false;
                }
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                ComplexTridiagonalMatrix m=new ComplexTridiagonalMatrix(matrix[0].length);
                m.setElement(0,0,matrix[1][0]);
                m.setElement(1,0,matrix[2][0]);
                for(int i=0;i<matrix[0].length-1;i++) {
                        m.setElement(i-1,i,matrix[0][i]);
                        m.setElement(i,i,matrix[1][i]);
                        m.setElement(i+1,i,matrix[2][i]);
                }
                m.setElement(matrix[0].length-2,matrix[0].length-1,matrix[0][matrix[0].length-1]);
                m.setElement(matrix[0].length-1,matrix[0].length-1,matrix[1][matrix[0].length-1]);
                return m;
        }
        /**
        * Returns a string representing this matrix.
        */
        public String toString() {
                StringBuffer buf=new StringBuffer(matrix.length*matrix[0].length);
                for(int j,i=0;i<rows();i++) {
                        for(j=0;j<columns();j++) {
                                buf.append(getElement(i,j).toString());
                                buf.append(' ');
                        }
                        buf.append('\n');
                }
                return buf.toString();
        }
        /**
        * Returns the real part of this complex matrix.
        * @return a double tridiagonal matrix
        */
        public DoubleMatrix real() {
                DoubleTridiagonalMatrix m=new DoubleTridiagonalMatrix(matrix[0].length);
                m.setElement(0,0,matrix[1][0].real());
                m.setElement(1,0,matrix[2][0].real());
                for(int i=0;i<matrix[0].length-1;i++) {
                        m.setElement(i-1,i,matrix[0][i].real());
                        m.setElement(i,i,matrix[1][i].real());
                        m.setElement(i+1,i,matrix[2][i].real());
                }
                m.setElement(matrix[0].length-2,matrix[0].length-1,matrix[0][matrix[0].length-1].real());
                m.setElement(matrix[0].length-1,matrix[0].length-1,matrix[1][matrix[0].length-1].real());
                return m;
        }
        /**
        * Returns the imaginary part of this complex matrix.
        * @return a double tridiagonal matrix
        */
        public DoubleMatrix imag() {
                DoubleTridiagonalMatrix m=new DoubleTridiagonalMatrix(matrix[0].length);
                m.setElement(0,0,matrix[1][0].imag());
                m.setElement(1,0,matrix[2][0].imag());
                for(int i=0;i<matrix[0].length-1;i++) {
                        m.setElement(i-1,i,matrix[0][i].imag());
                        m.setElement(i,i,matrix[1][i].imag());
                        m.setElement(i+1,i,matrix[2][i].imag());
                }
                m.setElement(matrix[0].length-2,matrix[0].length-1,matrix[0][matrix[0].length-1].imag());
                m.setElement(matrix[0].length-1,matrix[0].length-1,matrix[1][matrix[0].length-1].imag());
                return m;
        }
        /**
        * Returns an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public Complex getElement(int i, int j) {
                if(i>=0 && i<matrix[0].length && j>=0 && j<matrix[0].length) {
                        if(i-1==j)
                                return matrix[0][i];
                        else if(i==j)
                                return matrix[1][i];
                        else if(i+1==j)
                                return matrix[2][i];
                        else
                                return Complex.ZERO;
                } else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Sets the value of an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @param z a complex number
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public void setElement(int i, int j, Complex z) {
                if(i>=0 && i<matrix[0].length && j>=0 && j<matrix[0].length) {
                        if(i-1==j)
                                matrix[0][i]=z;
                        else if(i==j)
                                matrix[1][i]=z;
                        else if(i+1==j)
                                matrix[2][i]=z;
                        else
                                throw new MatrixDimensionException("Invalid element.");
                } else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Returns the trace.
        */
        public Complex trace() {
                Complex tr=matrix[1][0];
                for(int i=1;i<matrix[1].length;i++)
                        tr=ComplexMath.add(tr,matrix[1][i]);
                return tr;
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                double result=0.0, tmpResult;
                for(int i=0;i<matrix[0].length;i++) {
                        tmpResult=matrix[0][i].mod()+matrix[1][i].mod()+matrix[2][i].mod();
                        if (tmpResult >= result)
                                result = tmpResult;
                }
                return result;
        }
        /**
        * Returns the Frobenius norm.
        * @author Taber Smith
        */
        public double frobeniusNorm() {
                double result=0.0,tmp;
                for(int i=0;i<matrix[0].length;i++) {
                        tmp=matrix[0][i].mod();
                        result+=tmp*tmp;
                        tmp=matrix[1][i].mod();
                        result+=tmp*tmp;
                        tmp=matrix[2][i].mod();
                        result+=tmp*tmp;
                }
                return Math.sqrt(result);
        }
        /**
        * Returns the number of rows.
        */
        public int rows() {
                return matrix[0].length;
        }
        /**
        * Returns the number of columns.
        */
        public int columns() {
                return matrix[0].length;
        }
}

