package JSci.maths;

/**
* The ComplexVector class provides an object for encapsulating vectors containing complex numbers.
* @version 1.5
* @author Mark Hale
*/
public final class ComplexVector extends Vector {
        private Complex vector[];

        /**
        * Constructs an empty vector.
        * @param dim the dimension of the vector.
        */
        public ComplexVector(int dim) {
                vector=new Complex[dim];
        }
        /**
        * Constructs a vector by wrapping an array.
        * @param array an assigned value
        */
        public ComplexVector(Complex array[]) {
                vector=array;
        }
        /**
        * Constructs a 3D vector.
        * @param x x ordinate
        * @param y y ordinate
        * @param z z ordinate
        */
        public ComplexVector(Complex x, Complex y, Complex z) {
                vector=new Complex[3];
                vector[0]=x;
                vector[1]=y;
                vector[2]=z;
        }
        /**
        * Compares two complex vectors for equality.
        * @param a a complex vector
        */
        public boolean equals(Object a) {
                if(a!=null && (a instanceof ComplexVector) && vector.length==((ComplexVector)a).dimension()) {
                        ComplexVector cv=(ComplexVector)a;
                        for(int i=0;i<vector.length;i++) {
                                if(!vector[i].equals(cv.getComponent(i)))
                                        return false;
                        }
                        return true;
                } else
                        return false;
        }
        /**
        * Creates a clone of this vector.
        * @return a copy of this vector.
        */
        public Object clone() {
                Complex array[]=new Complex[vector.length];
                for(int i=0;i<vector.length;i++)
                        array[i]=(Complex)vector[i].clone();
                return new ComplexVector(array);
        }
        /**
        * Returns a comma delimited string representing the value of this vector.
        */
        public String toString() {
                StringBuffer buf=new StringBuffer(vector.length);
                int i;
                for(i=0;i<vector.length-1;i++) {
                        buf.append(vector[i].toString());
                        buf.append(',');
                }
                buf.append(vector[i].toString());
                return buf.toString();
        }
        /**
        * Returns a hashcode for this vector.
        */
        public int hashCode() {
                return (int)Math.exp(norm());
        }
        /**
        * Returns the real part of this complex vector.
        */
        public DoubleVector real() {
                double array[]=new double[vector.length];
                for(int i=0;i<vector.length;i++)
                        array[i]=vector[i].real();
                return new DoubleVector(array);
        }
        /**
        * Returns the imaginary part of this complex vector.
        */
        public DoubleVector imag() {
                double array[]=new double[vector.length];
                for(int i=0;i<vector.length;i++)
                        array[i]=vector[i].imag();
                return new DoubleVector(array);
        }
        /**
        * Returns a component of this vector.
        * @param n index of the vector component
        * @exception VectorDimensionException If attempting to access an invalid component.
        */
        public Complex getComponent(int n) {
                if(n>=0 && n<vector.length)
                        return vector[n];
                else
                        throw new VectorDimensionException("Invalid component.");
        }
        /**
        * Sets the value of a component of this vector.
        * @param n index of the vector component
        * @param z a complex number
        * @exception VectorDimensionException If attempting to access an invalid component.
        */
        public void setComponent(int n, Complex z) {
                if(n>=0 && n<vector.length)
                        vector[n]=z;
                else
                        throw new VectorDimensionException("Invalid component.");
        }
        /**
        * Returns the vector's dimension.
        */
        public int dimension() {
                return vector.length;
        }
        /**
        * Returns the l2-norm (magnitude).
        */
        public double norm() {
                Complex answer=vector[0];
                for(int i=1;i<vector.length;i++)
                        answer=ComplexMath.add(answer,ComplexMath.multiply(vector[i], ComplexMath.conjugate(vector[i])));
                return Math.sqrt(answer.real());
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                double infNorm = vector[0].mod();
                for (int i=1; i<vector.length; i++) {
                        if (vector[i].mod() > infNorm)
                                infNorm = vector[i].mod();
                }
                return infNorm;
        }
}

