package JSci.maths;

/**
* The DoubleDiagonalMatrix class provides an object for encapsulating diagonal matrices containing doubles.
* @version 1.5
* @author Mark Hale
*/
public final class DoubleDiagonalMatrix extends DoubleTridiagonalMatrix {
        /**
        * Constructs a matrix.
        */
        private DoubleDiagonalMatrix() {
                super();
        }
        /**
        * Constructs an empty matrix.
        * @param size the number of rows/columns
        */
        public DoubleDiagonalMatrix(int size) {
                this();
                matrix=new double[1][size];
        }
        /**
        * Constructs a matrix from an array.
        * @param array an assigned value
        * @exception MatrixDimensionException If the array is not square.
        */
        public DoubleDiagonalMatrix(double array[][]) {
                this(array.length);
                if(array.length==array[0].length) {
                        for(int i=0;i<matrix[0].length;i++)
                                matrix[0][i]=array[i][i];
                } else {
                        matrix=null;
                        throw new MatrixDimensionException("Array must be square.");
                }
        }
        /**
        * Constructs a matrix by wrapping an array containing the diagonal elements.
        * @param array an assigned value
        */
        public DoubleDiagonalMatrix(double array[]) {
                this();
                matrix=new double[1][];
                matrix[0]=array;
        }
        /**
        * Creates an identity matrix.
        * @param size the number of rows/columns
        */
        public static DoubleDiagonalMatrix identity(int size) {
                double array[]=new double[size];
                for(int i=0;i<size;i++)
                        array[i]=1.0;
                return new DoubleDiagonalMatrix(array);
        }
        /**
        * Compares two double diagonal matrices for equality.
        * @param m a double diagonal matrix
        */
        public boolean equals(Object m) {
                if(m!=null && (m instanceof DoubleDiagonalMatrix) &&
                matrix[0].length==((DoubleDiagonalMatrix)m).rows()) {
                        DoubleDiagonalMatrix ddm=(DoubleDiagonalMatrix)m;
                        for(int i=0;i<matrix[0].length;i++) {
                                if(matrix[0][i]!=ddm.getElement(i,i))
                                        return false;
                        }
                        return true;
                } else
                        return false;
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                double array[]=new double[matrix[0].length];
                for(int i=0;i<matrix[0].length;i++)
                        array[i]=matrix[0][i];
                return new DoubleDiagonalMatrix(array);
        }
        /**
        * Converts this matrix to an integer matrix.
        * @return an integer diagonal matrix
        */
        public IntegerMatrix toIntegerMatrix() {
                int array[]=new int[matrix[0].length];
                for(int i=0;i<matrix[0].length;i++)
                        array[i]=Math.round((float)matrix[0][i]);
                return new IntegerDiagonalMatrix(array);
        }
        /**
        * Converts this matrix to a complex matrix.
        * @return a complex diagonal matrix
        */
        public ComplexMatrix toComplexMatrix() {
                Complex array[]=new Complex[matrix[0].length];
                for(int i=0;i<matrix[0].length;i++)
                        array[i]=new Complex(matrix[0][i],0.0);
                return new ComplexDiagonalMatrix(array);
        }
        /**
        * Returns an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public double getElement(int i, int j) {
                if(i>=0 && i<matrix[0].length && j>=0 && j<matrix[0].length) {
                        if(i==j)
                                return matrix[0][i];
                        else
                                return 0.0;
                } else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Sets the value of an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @param x a number
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public void setElement(int i, int j, double x) {
                if(i>=0 && i<matrix[0].length && j>=0 && j<matrix[0].length && i==j)
                        matrix[0][i]=x;
                else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Returns true if this matrix is hermitian.
        */
        public boolean isHermitian() {
                return true;
        }
        /**
        * Returns true if this matrix is unitary.
        */
        public boolean isUnitary() {
                return identity(matrix[0].length).equals(MatrixMath.multiply(this,this));
        }
        /**
        * Returns the determinant.
        */
        public double determinant() {
                double det=matrix[0][0];
                for(int i=1;i<matrix[0].length;i++)
                        det*=matrix[0][i];
                return det;
        }
        /**
        * Returns the trace.
        */
        public double trace() {
                double tr=matrix[0][0];
                for(int i=1;i<matrix[0].length;i++)
                        tr+=matrix[0][i];
                return tr;
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                double result=0.0, tmpResult;
                for(int i=0;i<matrix[0].length;i++) {
                        tmpResult=Math.abs(matrix[0][i]);
                        if (tmpResult >= result)
                                result = tmpResult;
                }
                return result;
        }
        /**
        * Returns the Frobenius norm.
        * @author Taber Smith
        */
        public double frobeniusNorm() {
                double result=matrix[0][0]*matrix[0][0];
                for(int i=1;i<matrix[0].length;i++)
                        result+=matrix[0][i]*matrix[0][i];
                return Math.sqrt(result);
        }
        /**
        * Returns the number of rows.
        */
        public int rows() {
                return matrix[0].length;
        }
        /**
        * Returns the number of columns.
        */
        public int columns() {
                return matrix[0].length;
        }
}

