package JSci.maths;

/**
* The DoubleMatrix class provides an object for encapsulating matrices containing doubles.
* @version 1.5
* @author Mark Hale
*/
public class DoubleMatrix extends Matrix {
        /**
        * Array containing the elements of the matrix.
        */
        protected double matrix[][];
        /**
        * Constructs a matrix.
        */
        protected DoubleMatrix() {
                super();
        }
        /**
        * Constructs an empty matrix.
        * @param row the number of rows
        * @param col the number of columns
        */
        public DoubleMatrix(int row,int col) {
                this();
                matrix=new double[row][col];
        }
        /**
        * Constructs a matrix by wrapping an array.
        * @param array an assigned value
        */
        public DoubleMatrix(double array[][]) {
                this();
                matrix=array;
        }
        /**
        * Constructs a matrix from an array of vectors.
        * @param array an assigned value
        */
        public DoubleMatrix(DoubleVector array[]) {
                this(array.length,array[0].dimension());
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                matrix[i][j]=array[i].getComponent(j);
                }
        }
        /**
        * Compares two double matrices for equality.
        * @param m a double matrix
        */
        public boolean equals(Object m) {
                if(m!=null && (m instanceof DoubleMatrix) &&
                matrix.length==((DoubleMatrix)m).rows() && matrix[0].length==((DoubleMatrix)m).columns()) {
                        DoubleMatrix dm=(DoubleMatrix)m;
                        for(int j,i=0;i<matrix.length;i++) {
                                for(j=0;j<matrix[0].length;j++) {
                                        if(matrix[i][j]!=dm.getElement(i,j)) {
                                                return false;
                                        }
                                }
                        }
                        return true;
                } else {
                        return false;
                }
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                double array[][]=new double[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                array[i][j]=matrix[i][j];
                }
                return new DoubleMatrix(array);
        }
        /**
        * Returns a string representing this matrix.
        */
        public String toString() {
                StringBuffer buf=new StringBuffer(matrix.length*matrix[0].length);
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++) {
                                buf.append(matrix[i][j]);
                                buf.append(' ');
                        }
                        buf.append('\n');
                }
                return buf.toString();
        }
        /**
        * Returns a hashcode for this matrix.
        */
        public int hashCode() {
                return (int)Math.exp(infNorm());
        }
        /**
        * Converts this matrix to an integer matrix.
        * @return an integer matrix
        */
        public IntegerMatrix toIntegerMatrix() {
                int ans[][]=new int[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                ans[i][j]=Math.round((float)matrix[i][j]);
                }
                return new IntegerMatrix(ans);
        }
        /**
        * Converts this matrix to a complex matrix.
        * @return a complex matrix
        */
        public ComplexMatrix toComplexMatrix() {
                Complex ans[][]=new Complex[matrix.length][matrix[0].length];
                for(int j,i=0;i<matrix.length;i++) {
                        for(j=0;j<matrix[0].length;j++)
                                ans[i][j]=new Complex(matrix[i][j],0.0);
                }
                return new ComplexMatrix(ans);
        }
        /**
        * Returns true if this matrix is hermitian.
        */
        public boolean isHermitian() {
                return equals(MatrixMath.transpose(this));
        }
        /**
        * Returns an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public double getElement(int i, int j) {
                if(i>=0 && i<matrix.length && j>=0 && j<matrix[0].length)
                        return matrix[i][j];
                else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Sets the value of an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @param x a number
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public void setElement(int i, int j, double x) {
                if(i>=0 && i<matrix.length && j>=0 && j<matrix[0].length)
                        matrix[i][j]=x;
                else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                double result=0.0, tmpResult;
                for(int j,i=0;i<matrix.length;i++) {
                        tmpResult = 0.0;
                        for(j=0;j<matrix[0].length;j++)
                                tmpResult += Math.abs(matrix[i][j]);
                        if (tmpResult >= result)
                                result = tmpResult;
                }
                return result;
        }
        /**
        * Returns the Frobenius norm.
        * @author Taber Smith
        */
        public double frobeniusNorm() {
                double result=0.0;
                for(int j,i=0;i<matrix.length;i++)
                        for(j=0;j<matrix[0].length;j++)
                                result+=matrix[i][j]*matrix[i][j];
                return Math.sqrt(result);
        }
        /**
        * Returns the number of rows.
        */
        public int rows() {
                return matrix.length;
        }
        /**
        * Returns the number of columns.
        */
        public int columns() {
                return matrix[0].length;
        }
}

