package JSci.maths;

/**
* The DoubleTridiagonalMatrix class provides an object for encapsulating tridiagonal matrices containing doubles.
* @version 1.5
* @author Mark Hale
*/
public class DoubleTridiagonalMatrix extends DoubleSquareMatrix {
        /**
        * Constructs a matrix.
        */
        protected DoubleTridiagonalMatrix() {
                super();
        }
        /**
        * Constructs an empty matrix.
        * @param size the number of rows/columns
        */
        public DoubleTridiagonalMatrix(int size) {
                this();
                matrix=new double[3][size];
        }
        /**
        * Constructs a matrix from an array.
        * @param array an assigned value
        * @exception MatrixDimensionException If the array is not square.
        */
        public DoubleTridiagonalMatrix(double array[][]) {
                this(array.length);
                if(array.length==array[0].length) {
                        matrix[1][0]=array[0][0];
                        matrix[2][0]=array[1][0];
                        for(int i=0;i<array.length-1;i++) {
                                matrix[0][i]=array[i-1][i];
                                matrix[1][i]=array[i][i];
                                matrix[2][i]=array[i+1][i];
                        }
                        matrix[0][array.length-1]=array[array.length-2][array.length-1];
                        matrix[1][array.length-1]=array[array.length-1][array.length-1];
                } else {
                        matrix=null;
                        throw new MatrixDimensionException("The array is not square.");
                }
        }
        /**
        * Compares two double tridiagonal matrices for equality.
        * @param m a double tridiagonal matrix
        */
        public boolean equals(Object m) {
                if(m!=null && (m instanceof DoubleTridiagonalMatrix) &&
                matrix[0].length==((DoubleTridiagonalMatrix)m).rows()) {
                        DoubleTridiagonalMatrix dtm=(DoubleTridiagonalMatrix)m;
                        if(matrix[1][0]!=dtm.getElement(0,0))
                                return false;
                        if(matrix[2][0]!=dtm.getElement(1,0))
                                return false;
                        for(int i=1;i<matrix[0].length-1;i++) {
                                if(matrix[0][i]!=dtm.getElement(i-1,i))
                                        return false;
                                if(matrix[1][i]!=dtm.getElement(i,i))
                                        return false;
                                if(matrix[2][i]!=dtm.getElement(i+1,i))
                                        return false;
                        }
                        if(matrix[0][matrix[0].length-1]!=dtm.getElement(matrix[0].length-2,matrix[0].length-1))
                                return false;
                        if(matrix[1][matrix[0].length-1]!=dtm.getElement(matrix[0].length-1,matrix[0].length-1))
                                return false;
                        return true;
                } else {
                        return false;
                }
        }
        /**
        * Creates a clone of this matrix.
        * @return a copy of this matrix.
        */
        public Object clone() {
                DoubleTridiagonalMatrix m=new DoubleTridiagonalMatrix(matrix[0].length);
                m.setElement(0,0,matrix[1][0]);
                m.setElement(1,0,matrix[2][0]);
                for(int i=0;i<matrix[0].length-1;i++) {
                        m.setElement(i-1,i,matrix[0][i]);
                        m.setElement(i,i,matrix[1][i]);
                        m.setElement(i+1,i,matrix[2][i]);
                }
                m.setElement(matrix[0].length-2,matrix[0].length-1,matrix[0][matrix[0].length-1]);
                m.setElement(matrix[0].length-1,matrix[0].length-1,matrix[1][matrix[0].length-1]);
                return m;
        }
        /**
        * Returns a string representing this matrix.
        */
        public String toString() {
                StringBuffer buf=new StringBuffer(matrix.length*matrix[0].length);
                for(int j,i=0;i<rows();i++) {
                        for(j=0;j<columns();j++) {
                                buf.append(getElement(i,j));
                                buf.append(' ');
                        }
                        buf.append('\n');
                }
                return buf.toString();
        }
        /**
        * Converts this matrix to an integer matrix.
        * @return an integer tridiagonal matrix
        */
        public IntegerMatrix toIntegerMatrix() {
                IntegerTridiagonalMatrix m=new IntegerTridiagonalMatrix(matrix[0].length);
                m.setElement(0,0,Math.round((float)matrix[1][0]));
                m.setElement(1,0,Math.round((float)matrix[2][0]));
                for(int i=0;i<matrix[0].length-1;i++) {
                        m.setElement(i-1,i,Math.round((float)matrix[0][i]));
                        m.setElement(i,i,Math.round((float)matrix[1][i]));
                        m.setElement(i+1,i,Math.round((float)matrix[2][i]));
                }
                m.setElement(matrix[0].length-2,matrix[0].length-1,Math.round((float)matrix[0][matrix[0].length-1]));
                m.setElement(matrix[0].length-1,matrix[0].length-1,Math.round((float)matrix[1][matrix[0].length-1]));
                return m;
        }
        /**
        * Converts this matrix to a complex matrix.
        * @return a complex tridiagonal matrix
        */
        public ComplexMatrix toComplexMatrix() {
                ComplexTridiagonalMatrix m=new ComplexTridiagonalMatrix(matrix[0].length);
                m.setElement(0,0,new Complex(matrix[1][0],0.0));
                m.setElement(1,0,new Complex(matrix[2][0],0.0));
                for(int i=0;i<matrix[0].length-1;i++) {
                        m.setElement(i-1,i,new Complex(matrix[0][i],0.0));
                        m.setElement(i,i,new Complex(matrix[1][i],0.0));
                        m.setElement(i+1,i,new Complex(matrix[2][i],0.0));
                }
                m.setElement(matrix[0].length-2,matrix[0].length-1,new Complex(matrix[0][matrix[0].length-1],0.0));
                m.setElement(matrix[0].length-1,matrix[0].length-1,new Complex(matrix[1][matrix[0].length-1],0.0));
                return m;
        }
        /**
        * Returns an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public double getElement(int i, int j) {
                if(i>=0 && i<matrix[0].length && j>=0 && j<matrix[0].length) {
                        if(i-1==j)
                                return matrix[0][i];
                        else if(i==j)
                                return matrix[1][i];
                        else if(i+1==j)
                                return matrix[2][i];
                        else
                                return 0.0;
                } else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Sets the value of an element of the matrix.
        * @param i row index of the element
        * @param j column index of the element
        * @param x a number
        * @exception MatrixDimensionException If attempting to access an invalid element.
        */
        public void setElement(int i, int j, double x) {
                if(i>=0 && i<matrix[0].length && j>=0 && j<matrix[0].length) {
                        if(i-1==j)
                                matrix[0][i]=x;
                        else if(i==j)
                                matrix[1][i]=x;
                        else if(i+1==j)
                                matrix[2][i]=x;
                        else
                                throw new MatrixDimensionException("Invalid element.");
                } else
                        throw new MatrixDimensionException("Invalid element.");
        }
        /**
        * Returns the trace.
        */
        public double trace() {
                double tr=matrix[1][0];
                for(int i=1;i<matrix[1].length;i++)
                        tr+=matrix[1][i];
                return tr;
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                double result=0.0, tmpResult;
                for(int i=0;i<matrix[0].length;i++) {
                        tmpResult=Math.abs(matrix[0][i])+Math.abs(matrix[1][i])+Math.abs(matrix[2][i]);
                        if (tmpResult >= result)
                                result = tmpResult;
                }
                return result;
        }
        /**
        * Returns the Frobenius norm.
        * @author Taber Smith
        */
        public double frobeniusNorm() {
                double result=matrix[0][0]*matrix[0][0]+matrix[1][0]*matrix[1][0]+matrix[2][0]*matrix[2][0];
                for(int i=1;i<matrix[0].length;i++)
                        result+=matrix[0][i]*matrix[0][i]+matrix[1][i]*matrix[1][i]+matrix[2][i]*matrix[2][i];
                return Math.sqrt(result);
        }
        /**
        * Returns the number of rows.
        */
        public int rows() {
                return matrix[0].length;
        }
        /**
        * Returns the number of columns.
        */
        public int columns() {
                return matrix[0].length;
        }
}

