package JSci.maths;

/**
* The DoubleVector class provides an object for encapsulating vectors containing doubles.
* @version 1.5
* @author Mark Hale
*/
public final class DoubleVector extends Vector {
        private double vector[];

        /**
        * Constructs an empty vector.
        * @param dim the dimension of the vector.
        */
        public DoubleVector(int dim) {
                vector=new double[dim];
        }
        /**
        * Constructs a vector by wrapping an array.
        * @param array an assigned value
        */
        public DoubleVector(double array[]) {
                vector=array;
        }
        /**
        * Constructs a 3D vector.
        * @param x x coordinate
        * @param y y coordinate
        * @param z z coordinate
        */
        public DoubleVector(double x, double y, double z) {
                vector=new double[3];
                vector[0]=x;
                vector[1]=y;
                vector[2]=z;
        }
        /**
        * Compares two double vectors for equality.
        * @param a a double vector
        */
        public boolean equals(Object a) {
                if(a!=null && (a instanceof DoubleVector) && vector.length==((DoubleVector)a).dimension()) {
                        DoubleVector dv=(DoubleVector)a;
                        for(int i=0;i<vector.length;i++) {
                                if(vector[i]!=dv.getComponent(i))
                                        return false;
                        }
                        return true;
                } else
                        return false;
        }
        /**
        * Creates a clone of this vector.
        * @return a copy of this vector.
        */
        public Object clone() {
                double array[]=new double[vector.length];
                for(int i=0;i<vector.length;i++)
                        array[i]=vector[i];
                return new DoubleVector(array);
        }
        /**
        * Returns a comma delimited string representing the value of this vector.
        */
        public String toString() {
                StringBuffer buf=new StringBuffer(vector.length);
                int i;
                for(i=0;i<vector.length-1;i++) {
                        buf.append(vector[i]);
                        buf.append(',');
                }
                buf.append(vector[i]);
                return buf.toString();
        }
        /**
        * Returns a hashcode for this vector.
        */
        public int hashCode() {
                return (int)Math.exp(norm());
        }
        /**
        * Converts this vector to an integer vector.
        * @return an integer vector
        */
        public IntegerVector toIntegerVector() {
                int ans[]=new int[vector.length];
                for(int i=0;i<vector.length;i++)
                        ans[i]=(int) vector[i];
                return new IntegerVector(ans);
        }
        /**
        * Converts this vector to a complex vector.
        * @return a complex vector
        */
        public ComplexVector toComplexVector() {
                Complex ans[]=new Complex[vector.length];
                for(int i=0;i<vector.length;i++)
                        ans[i]=new Complex(vector[i],0.0);
                return new ComplexVector(ans);
        }
        /**
        * Returns a component of this vector.
        * @param n index of the vector component
        * @exception VectorDimensionException If attempting to access an invalid component.
        */
        public double getComponent(int n) {
                if(n>=0 && n<vector.length)
                        return vector[n];
                else
                        throw new VectorDimensionException("Invalid component.");
        }
        /**
        * Sets the value of a component of this vector.
        * @param n index of the vector component
        * @param x a number
        * @exception VectorDimensionException If attempting to access an invalid component.
        */
        public void setComponent(int n, double x) {
                if(n>=0 && n<vector.length)
                        vector[n]=x;
                else
                        throw new VectorDimensionException("Invalid component.");
        }
        /**
        * Returns the vector's dimension.
        */
        public int dimension() {
                return vector.length;
        }
        /**
        * Returns the ln-norm.
        */
        public double norm(int n) {
                double answer=Math.pow(vector[0],n);
                for(int i=1;i<vector.length;i++)
                        answer+=Math.pow(vector[i],n);
                return Math.pow(answer,1.0/n);
        }
        /**
        * Returns the l2-norm (magnitude).
        */
        public double norm() {
                double answer=vector[0]*vector[0];
                for(int i=1;i<vector.length;i++)
                        answer+=vector[i]*vector[i];
                return Math.sqrt(answer);
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                double infNorm=vector[0];
                for (int i=1;i<vector.length;i++) {
                        if(vector[i]>infNorm)
                                infNorm=vector[i];
                }
                return infNorm;
        }
}

