package JSci.maths;

/**
* The IntegerVector class provides an object for encapsulating vectors containing integers.
* @version 1.5
* @author Mark Hale
*/
public final class IntegerVector extends Vector {
        private int vector[];

        /**
        * Constructs an empty vector.
        * @param dim the dimension of the vector.
        */
        public IntegerVector(int dim) {
                vector=new int[dim];
        }
        /**
        * Constructs a vector by wrapping an array.
        * @param array an assigned value
        */
        public IntegerVector(int array[]) {
                vector=array;
        }
        /**
        * Constructs a 3D vector.
        * @param x x ordinate
        * @param y y ordinate
        * @param z z ordinate
        */
        public IntegerVector(int x, int y, int z) {
                vector=new int[3];
                vector[0]=x;
                vector[1]=y;
                vector[2]=z;
        }
        /**
        * Compares two integer vectors for equality.
        * @param a an integer vector
        */
        public boolean equals(Object a) {
                if(a!=null && (a instanceof IntegerVector) && vector.length==((IntegerVector)a).dimension()) {
                        IntegerVector iv=(IntegerVector)a;
                        for(int i=0;i<vector.length;i++) {
                                if(vector[i]!=iv.getComponent(i))
                                        return false;
                        }
                        return true;
                } else
                        return false;
        }
        /**
        * Creates a clone of this vector.
        * @return a copy of this vector.
        */
        public Object clone() {
                int array[]=new int[vector.length];
                for(int i=0;i<vector.length;i++)
                        array[i]=vector[i];
                return new IntegerVector(array);
        }
        /**
        * Returns a comma delimited string representing the value of this vector.
        */
        public String toString() {
                StringBuffer buf=new StringBuffer(vector.length);
                int i;
                for(i=0;i<vector.length-1;i++) {
                        buf.append(vector[i]);
                        buf.append(',');
                }
                buf.append(vector[i]);
                return buf.toString();
        }
        /**
        * Returns a hashcode for this vector.
        */
        public int hashCode() {
                return (int)Math.exp(norm());
        }
        /**
        * Converts this vector to a double vector.
        * @return a double vector
        */
        public DoubleVector toDoubleVector() {
                double ans[]=new double[vector.length];
                for(int i=0;i<vector.length;i++)
                        ans[i]=vector[i];
                return new DoubleVector(ans);
        }
        /**
        * Converts this vector to a complex vector.
        * @return a complex vector
        */
        public ComplexVector toComplexVector() {
                Complex ans[]=new Complex[vector.length];
                for(int i=0;i<vector.length;i++)
                        ans[i]=new Complex(vector[i],0.0);
                return new ComplexVector(ans);
        }
        /**
        * Returns a component of this vector.
        * @param n index of the vector component
        * @exception VectorDimensionException If attempting to access an invalid component.
        */
        public int getComponent(int n) {
                if(n>=0 && n<vector.length)
                        return vector[n];
                else
                        throw new VectorDimensionException("Invalid component.");
        }
        /**
        * Sets the value of a component of this vector.
        * @param n index of the vector component
        * @param x an integer
        * @exception VectorDimensionException If attempting to access an invalid component.
        */
        public void setComponent(int n, int x) {
                if(n>=0 && n<vector.length)
                        vector[n]=x;
                else
                        throw new VectorDimensionException("Invalid component.");
        }
        /**
        * Returns the vector's dimension.
        */
        public int dimension() {
                return vector.length;
        }
        /**
        * Returns the ln-norm.
        */
        public double norm(int n) {
                double answer=Math.pow(vector[0],n);
                for(int i=1;i<vector.length;i++)
                        answer+=Math.pow(vector[i],n);
                return Math.pow(answer,1.0/n);
        }
        /**
        * Returns the l2-norm (magnitude).
        */
        public double norm() {
                int answer=vector[0]*vector[0];
                for(int i=1;i<vector.length;i++)
                        answer+=vector[i]*vector[i];
                return Math.sqrt(answer);
        }
        /**
        * Returns the l(infinity)-norm.
        * @author Taber Smith
        */
        public double infNorm() {
                int infNorm = vector[0];
                for (int i=1; i<vector.length; i++) {
                        if (vector[i] > infNorm)
                                infNorm = vector[i];
                }
                return infNorm;
        }
}

