package JSci.maths;

/**
* The numerical math library.
* This class cannot be subclassed or instantiated because all methods are static.
* @version 0.8
* @author Mark Hale
*/
public final class NumericalMath extends Object {
        private NumericalMath() {}

        /**
        * Uses the Euler method to solve an ODE.
        * @param y an array to be filled with y values, set y[0] to initial condition
        * @param func -dy/dt
        * @param dt step size
        * @return y
        */
        public static double[] euler(double y[],Mapping func,double dt) {
                for(int i=0;i<y.length-1;i++) {
                        y[i+1]=y[i]-dt*func.map(y[i]);
                }
                return y;
        }
        /**
        * Uses the Leap-Frog method to solve an ODE.
        * @param y an array to be filled with y values, set y[0],y[1] to initial conditions
        * @param func -dy/dt
        * @param dt step size
        * @return y
        */
        public static double[] leapFrog(double y[],Mapping func,double dt) {
                for(int i=1;i<y.length-1;i++) {
                        y[i+1]=y[i-1]-2*dt*func.map(y[i]);
                }
                return y;
        }
        /**
        * Uses the Runge-Kutta method to solve an ODE.
        * @param y an array to be filled with y values, set y[0] to initial condition
        * @param func -dy/dt
        * @param dt step size
        * @return y
        */
        public static double[] rungeKutta(double y[],Mapping func,double dt) {
                double tmp;
                for(int i=0;i<y.length-1;i++) {
                        tmp=y[i]-dt/2*func.map(y[i]);
                        y[i+1]=y[i]-dt*func.map(tmp);
                }
                return y;
        }
        /**
        * Numerical integration using the trapezium rule.
        * @param N the number of strips to use
        * @param func a function
        * @param a the first ordinate
        * @param b the last ordinate
        */
        public static double trapezium(int N,Mapping func,double a,double b) {
                double A=0,x=a,h=(b-a)/N;
                for(int i=0;i<N;i++) {
                        A+=func.map(x)+func.map(x+h);
                        x+=h;
                }
                return A*h/2;
        }
        /**
        * Numerical integration using Simpson's rule.
        * @param N the number of strip pairs to use
        * @param func a function
        * @param a the first ordinate
        * @param b the last ordinate
        */
        public static double simpson(int N,Mapping func,double a,double b) {
                double Ao=0,Ae=0,x=a,h=(b-a)/(2*N);
                for(int i=0;i<N-1;i++) {
                        Ao+=func.map(x+h);
                        Ae+=func.map(x+2*h);
                        x+=2*h;
                }
                Ao+=func.map(x+h);
                return h/3*(func.map(a)+4*Ao+2*Ae+func.map(b));
        }
        /**
        * Numerical integration using the Richardson extrapolation.
        * @param N the number of strip pairs to use (lower value)
        * @param func a function
        * @param a the first ordinate
        * @param b the last ordinate
        */
        public static double richardson(int N,Mapping func,double a,double b) {
                double Aa,Aao=0,Aae=0,Ab,Abo=0,Abe=0,x=a;
                double ha=(b-a)/(2*N);
                double hb=ha/2;
                for(int i=0;i<N-1;i++) {
                        Aao+=func.map(x+ha);
                        Aae+=func.map(x+2*ha);
                        Abo+=func.map(x+hb);
                        Abe+=func.map(x+2*hb);
                        Abo+=func.map(x+3*hb);
                        Abe+=func.map(x+4*hb);
                        x+=2*ha;
                }
                Aao+=func.map(x+ha);
                Abo+=func.map(x+hb);
                Abe+=func.map(x+2*hb);
                Abo+=func.map(x+3*hb);
                Aa=ha/3*(func.map(a)+4*Aao+2*Aae+func.map(b));
                Ab=hb/3*(func.map(a)+4*Abo+2*Abe+func.map(b));
                return (16*Ab-Aa)/15;
        }
        /**
        * Numerical integration using the Gaussian integration formula (4 points).
        * @param N the number of strips to use
        * @param func a function
        * @param a the first ordinate
        * @param b the last ordinate
        */
        public static double gaussian4(int N,Mapping func,double a,double b) {
                int n,i;
                double A=0,h=(b-a)/N,h2=h/2;
                double zeros[]=new double[4];
                double coeffs[]=new double[4];
                zeros[2]=0.339981043584856264802665759103;
                zeros[3]=0.861136311594052575223946488893;
                zeros[0]=-zeros[3];
                zeros[1]=-zeros[2];
                coeffs[0]=coeffs[3]=0.347854845137453857373063949222;
                coeffs[1]=coeffs[2]=0.652145154862546142626936050778;
                for(n=0;n<N;n++) {
                        for(i=0;i<zeros.length;i++)
                                A+=coeffs[i]*func.map(a+(zeros[i]+1)*h2);
                        a+=h;
                }
                return A*h2;
        }
        /**
        * Numerical integration using the Gaussian integration formula (8 points).
        * @param N the number of strips to use
        * @param func a function
        * @param a the first ordinate
        * @param b the last ordinate
        */
        public static double gaussian8(int N,Mapping func,double a,double b) {
                int n,i;
                double A=0,h=(b-a)/N,h2=h/2;
                double zeros[]=new double[8];
                double coeffs[]=new double[8];
                zeros[4]=0.183434642495649804939476142360;
                zeros[5]=0.525532409916328985817739049189;
                zeros[6]=0.796666477413626739591553936476;
                zeros[7]=0.960289856497536231683560868569;
                zeros[0]=-zeros[7];
                zeros[1]=-zeros[6];
                zeros[2]=-zeros[5];
                zeros[3]=-zeros[4];
                coeffs[0]=coeffs[7]=0.101228536290376259152531354310;
                coeffs[1]=coeffs[6]=0.222381034453374470544355994426;
                coeffs[2]=coeffs[5]=0.313706645877887287337962201987;
                coeffs[3]=coeffs[4]=0.362683783378361982965150449277;
                for(n=0;n<N;n++) {
                        for(i=0;i<zeros.length;i++)
                                A+=coeffs[i]*func.map(a+(zeros[i]+1)*h2);
                        a+=h;
                }
                return A*h2;
        }
        /**
        * Numerical differentiation.
        * @param N the number of points to use
        * @param func a function
        * @param a the first ordinate
        * @param b the last ordinate
        */
        public static double[] differentiate(int N,Mapping func,double a,double b) {
                double diff[]=new double[N];
                double x=a,dx=(b-a)/N,dx2=dx/2;
                for(int i=0;i<N;i++) {
                        diff[i]=(func.map(x+dx2)-func.map(x-dx2))/dx;
                        x+=dx;
                }
                return diff;
        }
        /**
        * The Metropolis algorithm.
        * @param list an array to be filled with values distributed according to func, set list[0] to initial value
        * @param func distribution function
        * @param dx step size
        * @return list
        */
        public static double[] metropolis(double list[],Mapping func,double dx) {
                for(int i=0;i<list.length-1;i++) {
                        list[i+1]=list[i]+dx*(2*Math.random()-1);
                        if(func.map(list[i+1])/func.map(list[i])<Math.random())
                                list[i+1]=list[i];
                }
                return list;
        }
}

