package JSci.physics.quantum;

import JSci.maths.*;

/**
* The Quantum math library.
* This class cannot be subclassed or instantiated because all methods are static.
* @version 1.5
* @author Mark Hale
*/
public final class QuantumMath extends Object {
        private QuantumMath() {}

// MULTIPLICATION

        /**
        * Returns the multiplication of a ket vector by a bra vector.
        * @param bra a bra vector
        * @param ket a ket vector
        * @exception VectorDimensionException If the vectors have different dimensions.
        */
        public static Complex multiply(BraVector bra, KetVector ket) {
                int ketDim=ket.dimension();
                if(ketDim==bra.dimension()) {
                        ComplexVector b=bra.getRepresentation();
                        ComplexVector k=ket.getRepresentation();
                        Complex answer=ComplexMath.multiply(b.getComponent(0), k.getComponent(0));
                        for(int i=1;i<ketDim;i++)
                                answer=ComplexMath.add(answer,ComplexMath.multiply(b.getComponent(i), k.getComponent(i)));
                        return answer;
                } else
                        throw new VectorDimensionException("Vectors have different dimensions.");
        }
        /**
        * Returns the multiplication of a bra vector by a ket vector.
        * @param ket a ket vector
        * @param bra a bra vector
        * @exception VectorDimensionException If the vectors have different dimensions.
        */
        public static Operator multiply(KetVector ket, BraVector bra) {
                int ketDim=ket.dimension();
                if(ketDim==bra.dimension()) {
                        ComplexVector b=bra.getRepresentation();
                        ComplexVector k=ket.getRepresentation();
                        Complex array[][]=new Complex[ketDim][ketDim];
                        for(int j,i=0;i<ketDim;i++) {
                                array[i][0]=ComplexMath.multiply(k.getComponent(i), b.getComponent(0));
                                for(j=1;j<ketDim;j++)
                                        array[i][j]=ComplexMath.multiply(k.getComponent(i), b.getComponent(j));
                        }
                        return new Operator(new ComplexSquareMatrix(array));
                } else
                        throw new VectorDimensionException("Vectors have different dimensions.");
        }
        /**
        * Returns the multiplication of an operator by a bra vector.
        * (Acts an operator on a bra vector).
        * @param bra a bra vector
        * @param op an operator
        * @exception DimensionException If the operator and vector have different dimensions.
        */
        public static BraVector multiply(BraVector bra, Operator op) {
                int opDim=op.dimension();
                if(opDim==bra.dimension()) {
                        ComplexVector b=bra.getRepresentation();
                        ComplexSquareMatrix o=op.getRepresentation();
                        Complex tmp,array[]=new Complex[opDim];
                        for(int j,i=0;i<opDim;i++) {
                                tmp=ComplexMath.multiply(b.getComponent(0), o.getElement(0,i));
                                for(j=1;j<opDim;j++)
                                        tmp=ComplexMath.add(tmp, ComplexMath.multiply(b.getComponent(j), o.getElement(j,i)));
                                array[i]=tmp;
                        }
                        return new BraVector(new ComplexVector(array));
                } else
                        throw new DimensionException("Operator and vector have different dimensions.");
        }
        /**
        * Returns the multiplication of a ket vector by an operator.
        * (Acts an operator on a ket vector).
        * @param op an operator
        * @param ket a ket vector
        * @exception DimensionException If the operator and vector have different dimensions.
        */
        public static KetVector multiply(Operator op, KetVector ket) {
                int opDim=op.dimension();
                if(opDim==ket.dimension()) {
                        ComplexVector k=ket.getRepresentation();
                        ComplexSquareMatrix o=op.getRepresentation();
                        Complex tmp,array[]=new Complex[opDim];
                        for(int j,i=0;i<opDim;i++) {
                                tmp=ComplexMath.multiply(o.getElement(i,0), k.getComponent(0));
                                for(j=1;j<opDim;j++)
                                        tmp=ComplexMath.add(tmp, ComplexMath.multiply(o.getElement(i,j), k.getComponent(j)));
                                array[i]=tmp;
                        }
                        return new KetVector(new ComplexVector(array));
                } else
                        throw new DimensionException("Operator and vector have different dimensions.");
        }
        /**
        * Returns the multiplication of two operators.
        * @param op1 an operator
        * @param op2 an operator
        * @exception MatrixDimensionException If the operators have different dimensions.
        */
        public static Operator multiply(Operator op1, Operator op2) {
                return new Operator(MatrixMath.multiply(op1.getRepresentation(),op2.getRepresentation()));
        }

        /**
        * Returns the commutator [A,B].
        * @param A an operator
        * @param B an operator
        */
        public static Operator commutator(Operator A, Operator B) {
                ComplexSquareMatrix AB=multiply(A,B).getRepresentation();
                ComplexSquareMatrix BA=multiply(B,A).getRepresentation();
                return new Operator(MatrixMath.subtract(AB,BA));
        }
        /**
        * Returns the anticommutator {A,B}.
        * @param A an operator
        * @param B an operator
        */
        public static Operator anticommutator(Operator A, Operator B) {
                ComplexSquareMatrix AB=multiply(A,B).getRepresentation();
                ComplexSquareMatrix BA=multiply(B,A).getRepresentation();
                return new Operator(MatrixMath.add(AB,BA));
        }

// EXPECTATION VALUES

        /**
        * Returns the expectation value.
        * @param op an operator
        * @param ket a ket vector
        * @exception DimensionException If the operator and vector have different dimensions.
        */
        public static Complex expectation(Operator op, KetVector ket) {
                return multiply(ket.toBraVector(),multiply(op,ket));
        }
        /**
        * Returns the expectation value.
        * @param dm a density matrix
        * @param op an operator
        * @exception MatrixDimensionException If the operator and matrix have different dimensions.
        */
        public static Complex expectation(DensityMatrix dm, Operator op) {
                return multiply(dm,op).trace();
        }

// PROBABILITIES

        /**
        * Returns the probability.
        * @param p a projector
        * @param ket a ket vector
        * @exception DimensionException If the projector and vector have different dimensions.
        */
        public static Complex probability(Projector p, KetVector ket) {
                return multiply(ket.toBraVector(),multiply(p,ket));
        }
        /**
        * Returns the probability.
        * @param dm a density matrix
        * @param p a projector
        * @exception MatrixDimensionException If the projector and matrix have different dimensions.
        */
        public static Complex probability(DensityMatrix dm, Projector p) {
                return multiply(dm,p).trace();
        }
}

