package JSci.physics.relativity;

import JSci.maths.*;

/**
* The CoVector class provides an object for encapsulating covariant vectors.
* @version 1.0
* @author Mark Hale
*/
public final class CoVector extends Vector {
        private double vector[];

        /**
        * Constructs a covariant vector.
        * @param s a scalar
        * @param v1 1st 3-vector component
        * @param v2 2nd 3-vector component
        * @param v3 3rd 3-vector component
        */
        public CoVector(double s, double v1, double v2, double v3) {
                vector=new double[4];
                vector[0]=s;
                vector[1]=v1;
                vector[2]=v2;
                vector[3]=v3;
        }
        /**
        * Compares two covariant vectors for equality.
        * @param a a covariant vector
        */
        public boolean equals(Object a) {
                if(a!=null && (a instanceof CoVector)) {
                        CoVector v=(CoVector)a;
                        if(vector[0]==v.getComponent(0) && vector[1]==v.getComponent(1)
                                && vector[2]==v.getComponent(2) && vector[3]==v.getComponent(3))
                                return true;
                }
                return false;
        }
        /**
        * Creates a clone of this covariant vector.
        * @return a copy of this covariant vector.
        */
        public Object clone() {
                return new CoVector(vector[0],vector[1],vector[2],vector[3]);
        }
        /**
        * Returns a comma delimited string representing the value of this vector.
        */
        public String toString() {
                return new String(vector[0]+","+vector[1]+","+vector[2]+","+vector[3]);
        }
        /**
        * Returns a hashcode for this covariant vector.
        */
        public int hashCode() {
                return (int)Math.exp(norm());
        }
        /**
        * Returns a component of this vector.
        * @param n index of the vector component
        * @exception VectorDimensionException If attempting to access an invalid component.
        */
        public double getComponent(int n) {
                if(n>=0 && n<4)
                        return vector[n];
                else
                        throw new VectorDimensionException("Invalid component.");
        }
        /**
        * Returns the vector's dimension.
        */
        public int dimension() {
                return 4;
        }
        /**
        * Returns the norm (invariant).
        */
        public double norm() {
                return Math.sqrt(vector[0]*vector[0]-vector[1]*vector[1]
                        -vector[2]*vector[2]-vector[3]*vector[3]);
        }
}

