#!/usr/npac/bin/perl -w

#############################################################################
#  
#   File:    mailform.pl
#
#   Author:  Tom Scavo <trscavo@npac.syr.edu>
#
#   This perl script is a web-based mail utility.  If invoked with
#   no parameters, a simple HTML form is returned to the browser.
#   Otherwise the script will process a query string containing 
#   any number of keyword-value pairs.  An e-mail message is 
#   constructed from this input and sent to the specified recipient.
#
#   One of the keywords must be:
#
#     _mailTo
#
#   whose value is the address of the recipient of the e-mail message.
#   Here are some examples of HTML form elements satisfying this requirement:
#
#     <INPUT TYPE=HIDDEN NAME=_mailTo VALUE="trscavo@syr.edu">
#     <INPUT TYPE=HIDDEN NAME=_mailTo VALUE="trscavo@syr.edu (Tom Scavo)">
#     <INPUT TYPE=HIDDEN NAME=_mailTo 
#            VALUE="&quot;Tom Scavo&quot; &lt;trscavo@syr.edu&gt;">
#
#   The following keywords are optional, but processed specially:
#
#     _mailFrom
#     _mailSubject
#     _mailCc
#     _mailBcc
#     _mailConfirmURL
#     _mailArchive
#
#   The purpose of the first four of these keywords should be obvious.
#
#   The keyword '_mailConfirmURL' gives the URL of an HTML page
#   confirming that a message has been sent.  If this keyword is 
#   omitted, a default confirmation page will be returned.
#
#   The keyword '_mailArchive' is used to specify a file archive.
#   For example,
#
#     <INPUT TYPE=HIDDEN NAME=_mailArchive VALUE="mail.txt">
#
#   would cause all messages processed by mailform to be archived
#   in a file called "mail.txt".
#
#   All other keyword-value pairs are sent to the recipient specified
#   in '_mailTo', after which the user is notified whether or not the 
#   mailing was successful.
#
#############################################################################

#############################################################################
#  
#   Initialization
#
#############################################################################

use strict;
# Import a standard set of methods from CGI.pm (see
# page 187 of the Llama book for details):
use CGI qw(:standard);

# Jeffrey Friedl's e-mail pattern matcher (see Chapter 7 of
# his book "Mastering Regular Expressions", which explains
# this 6,598 byte-long regex in detail!):
require "email-opt.pl";

my @keywords   = ('_mailTo', 
                  '_mailFrom', 
                  '_mailSubject',
                  '_mailCc', 
                  '_mailBcc',
                  '_mailConfirmURL',
                  '_mailArchive');
                  
my $maintainer =  'trscavo@npac.syr.edu';     # script maintainer
my $sendmail   =  '/usr/lib/sendmail -t -n';  # sendmail deamon
# my $sendmail   =  '/usr/bin/more';            # for debugging

# Other global variables:
my (
  $BODY,      # message body constructed by BuildBody()
  $ORIGIN,    # originating host of sender
  $MESSAGE    # error messages
);

#############################################################################
#  
#   Main routines
#
#############################################################################

if ( param() ) {
  if ( ValidData() ) {
    GetOrigin();
    SendData();
  } else {
    SendError();
  }
} else {
  PrintForm();
}

#############################################################################
#  
#   Subroutines
#
#############################################################################

# Try to determine the host of origin:
sub GetOrigin {
  my ($remote_address, @subnet_numbers, $packed_address, $remote_host);
  
  # Convert IP address into hostname:
  if ( defined($ENV{'REMOTE_ADDR'}) ) {
    $remote_address = $ENV{'REMOTE_ADDR'};
    @subnet_numbers = split(/\./, $remote_address);
    $packed_address = pack("C4", @subnet_numbers);
    ($remote_host) = gethostbyaddr($packed_address, 2);
    $ORIGIN = defined($remote_host) ? $remote_host : $remote_address;
  } else {
    $ORIGIN = "UNKNOWN";
  }
  
  return 1;
}

# A boolean function that checks for valid input data:
sub ValidData {
  my ( $toAddress, $fromAddress );
  $MESSAGE = "";
  my $isOkay = 1;
  my $pat = $email::mailbox;           # defined in email-opt.pl
  if ( $toAddress = param('_mailTo') ) {
    if ( $toAddress !~ m/^$pat$/xo ) {
      $MESSAGE .= "Invalid e-mail address: $toAddress<BR>\n";
      $isOkay = 0;
    }
  } else {
    $MESSAGE .= qq{"To:" address omitted!<BR>\n};
    $isOkay = 0;
  }
  if ( $fromAddress = param('_mailFrom') ) {
    if ( $fromAddress !~ m/^$pat$/xo ) {
      $MESSAGE .= "Invalid e-mail address: $fromAddress<BR>\n";
      $isOkay = 0;
    }
  }
  
  return $isOkay;
}

# Invoke the mail deamon:
sub SendData {
  if ( open(MAIL, "|$sendmail") ) {
    print MAIL "To: ", param('_mailTo'), "\n";
    print MAIL "From: ", param('_mailFrom'), "\n";
    print MAIL "Subject: ", param('_mailSubject'), "\n";
    print MAIL "Cc: ", param('_mailCc'), "\n";
    print MAIL "Bcc: ", param('_mailBcc'), "\n";
    print MAIL "X-Remote-Host: $ORIGIN\n";
    BuildBody();
    print MAIL "\n$BODY";
    close(MAIL);
    UpdateArchive() &&
    SendConfirm();
    return 1;
  } else {
    $MESSAGE = "$sendmail could not be opened!<BR>\n";
    SendError();
    return 0;
  }
}

# Optionally update a mail archive:
sub UpdateArchive {
  if ( defined(param('_mailArchive')) ) {
    if ( open(ARCHIVE, ">>". param('_mailArchive')) ) {
      print ARCHIVE "Date:  " . `date`;
      print ARCHIVE "To: ", param('_mailTo'), "\n";
      print ARCHIVE "From: ", param('_mailFrom'), "\n" if ( defined(param('_mailFrom')) );
      print ARCHIVE "Subject: ", param('_mailSubject'), "\n" if ( defined(param('_mailSubject')) );
      print ARCHIVE "Cc: ", param('_mailCc'), "\n" if ( defined(param('_mailCc')) );
      print ARCHIVE "Bcc: ", param('_mailBcc'), "\n" if ( defined(param('_mailBcc')) );
      print ARCHIVE "X-Remote-Host: $ORIGIN\n";
      print ARCHIVE "\n$BODY";
      print ARCHIVE "-------------------------------------------------\n\n";
      close(ARCHIVE);
    } else {
      $MESSAGE = param('_mailArchive') ." could not be opened!<BR>\n";
      SendError();
      return 0;
    }
  }
  return 1;
}

#############################################################################
#  
#   Output
#
#############################################################################

# Print a simple HTML input form:
sub PrintForm {

  # Print MIME header:
  print header(-type=>'text/html');
  
  # Print first few lines of the HTML document:
  print start_html(
          -title=>'Universal Mail Form',
          -BGCOLOR=>'white'
        ), "\n";
  
  # Print a heading:
  print h2('Mail Form'), "\n";
  
  # Print <FORM> tag:
  print start_form();
                
  # Print a textfield for the recipient's address:
  print p( "To:\n",
           textfield(
             -name=>'_mailTo',
             -size=>30
           )
        ), "\n";
        
  # Print a textfield for the sender's address:
  print p( "From:\n",
           textfield(
             -name=>'_mailFrom',
             -size=>30
           )
        ), "\n";
        
  # Print a textfield for the subject:
  print p( "Subject:\n",
           textfield(
             -name=>'_mailSubject',
             -size=>30
           )
        ), "\n";
        
  # Print a textarea for e-mail message:
  print "Type your message:\n";
  print p( textarea(
             -name=>'message',
             -rows=>10, 
             -columns=>60
           )
        ), "\n";
  
  # Print submit and reset buttons:
  print p( submit('Send'), 
           reset('Erase') 
        ), "\n";
  
  # Print </FORM>:
  print end_form(), "\n";
  
  # Print last few lines of HTML document:
  print end_html(), "\n";

}

# Redirect the browser to a confirmation page or print a
# default confirmation page:
sub SendConfirm {
  if ( defined(param('_mailConfirmURL')) ) {
    print redirect(param('_mailConfirmURL'));
    return 1;
  }
  
  # Print MIME header:
  print header(-type=>'text/html');
  
  # Print first few lines of the HTML document:
  print start_html(
          -title=>'Mail Form Confirmation Notice',
          -BGCOLOR=>'white'
        ), "\n";
  
  # Print a heading:
  print h1('Mail Form Notice!'), "\n";
  
  # Print HTML body:
  print 'Your message has been sent to', "\n";
  print p( param('_mailTo') ), "\n";
  print 'Thank you for using mailform!', "\n";
  
  # Print last few lines of HTML document:
  print end_html(), "\n";
  
  return 1;
}

# Print an error page:
sub SendError {
  # Print MIME header:
  print header(-type=>'text/html');
  
  # Print first few lines of the HTML document:
  print start_html(
          -title=>'Mail Form Error Notice',
          -BGCOLOR=>'white'
        ), "\n";
  
  # Print a heading:
  print h1('Mail Form Error!'), "\n";
  
  # Print HTML body:
  print p( $MESSAGE ), "\n";
  print 'Please contact', "\n";
  print "<A HREF=\"mailto:$maintainer\">the maintainer of this script</A>.\n";
  
  # Print last few lines of HTML document:
  print end_html(), "\n";

  return 1;
}

#############################################################################
#  
#   Utilities
#
#############################################################################

# Construct mail message body:
sub BuildBody {
  my $val;
  $BODY = "";
  foreach ( param() ) {
    if ( ! IsSpecial($_) ) {
      $val = param($_);
      $BODY .= "$_: $val\n";
    }
  }
}

# Test for array membership:
sub IsSpecial {
  my ($special, $keyword);
  
  $special = 0;
  foreach $keyword ( @keywords ) {
    if ($_[0] eq $keyword) {
      $special = 1;
      last;
    }
  }
  
  return $special;
}
