/*
 *  File:  DrawCanvas.java
 *
 *  DrawCanvas class, a subclass of java.awt.Canvas
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.awt.*;
import java.awt.event.*;

public class DrawCanvas extends Canvas {

  // Note:  All instance variables should be private in this
  // case, but a longstanding bug with respect to inner classes in
  // Java 1.1 prevents this.  (See page 103 of the "Tiger" book.) 
  
  // Instance variables:
  String tools[] = { "Rectangle", "RoundRect", "Oval", "Line" };
  String tool = tools[0];
  boolean fill = true;
  int numObj = 0;
  DrawableObject curObj;
  final int maxObj = 100;  // maximum number of objects
  DrawableObject objList[] = new DrawableObject[ maxObj ];
  Graphics g;

  // Accessors and mutators:
  public void setMode( String new_mode ) {
    tool = new_mode;
  }
  public void setFill( boolean new_fill ) {
    fill = new_fill;
  }
  public int getNumObj() { return numObj; }
  public String getTool() { return tool; }
  public String[] getTools() { return tools; }

  // Constructor:
  public DrawCanvas() {
    
    setBackground(Color.blue);  // default canvas color
    
    // The following class, a type of inner class called a 
    // "local class", is a subclass of MouseAdapter.  Only 
    // two methods from the MouseAdapter class are being 
    // overridden:
    class MouseHandler extends MouseAdapter {
    
      public void mousePressed( MouseEvent e ) {
        int x = e.getX();
        int y = e.getY();
        
        if ( tool.equals( "Line" ) ) 
          curObj = new DrawableLine( x, y, x, y );
        else if ( tool.equals( "Rectangle" ) ) 
          curObj = new DrawableRectangle( x, y, x, y );
        else if ( tool.equals( "RoundRect" ) ) 
          curObj = new DrawableRoundRect( x, y, x, y );
        else if ( tool.equals( "Oval" ) ) 
          curObj = new DrawableOval( x, y, x, y );
        // else throw an exception
        
        // Set draw mode for rubberbanding:
        curObj.setFill( false );
      }

      public void mouseReleased( MouseEvent e ) {
        g = getGraphics();
        
        // Since XOR mode is still in effect, the following
        // statement will erase the current object:
        curObj.paint(g);
        
        int X = e.getX(), Y = e.getY();
        // Ignore a degenerate object (i.e., a point):
        if ( !( X == curObj.getX() && Y == curObj.getY() ) ) {
          // Turn XOR mode off and paint the final object:
          g.setPaintMode();
          curObj.setEndpoint( X, Y );
          curObj.setFill( fill );
          curObj.paint(g);
          // Save the first maxObj number of objects:
          if ( numObj < maxObj )
            objList[ numObj++ ] = curObj;
        }
      }
      
    }  // end class MouseHandler
    
    // This local class overrides a method from the 
    // MouseMotionAdapter class:
    class MouseMotionHandler extends MouseMotionAdapter {
    
      public void mouseDragged( MouseEvent e ) {
        g = getGraphics();
        
        // Erase the current object:
        g.setXORMode( Color.white );
        curObj.paint(g);
        
        // Paint a new object:
        curObj.setEndpoint( e.getX(), e.getY() );
        curObj.paint(g);
      }
      
    }  // end class MouseMotionHandler
    
    // Listen for MouseEvents:
    addMouseListener( new MouseHandler() );
    addMouseMotionListener( new MouseMotionHandler() );

  }  // end DrawCanvas constructor
  
  public void clear() {
    numObj = 0;
    repaint();
  }

  // Override Canvas paint method:
  public void paint( Graphics g ) {
    for ( int i = 0; i < numObj; i++ ) {
      objList[i].paint(g);
    }
  }
  
}  // end class DrawCanvas
