/*
 *  File:  DrawablePolygon.java
 *
 *  Custom polygon class for Java 1.1
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.awt.Polygon;
import java.awt.Color;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Component;

/*
 *  DrawablePolygon inherits variables xpoints[], ypoints[], 
 *  npoints and bounds, and numerous methods from Polygon
 *  
 */

public class DrawablePolygon extends Polygon implements Drawable {

   // instance variable:
   private Color color;

   // DrawablePolygon constructor #1:
   public DrawablePolygon() {

      // invoke the no-argument constructor of the superclass:
      super();

   }

   // DrawablePolygon constructor #2 (for compatibility with Polygon):
   public DrawablePolygon( int[] xpoints, int[] ypoints, int npoints ) {
       
      // invoke a constructor of the superclass:
      super( xpoints, ypoints, npoints );
      
   }

   // DrawablePolygon constructor #3:
   public DrawablePolygon( Point[] points, int npoints ) {
      
      // invoke DrawablePolygon constructor #1:
      this();

      for ( int i = 0; i < npoints; i++ ) {
         this.addPoint( points[i] );        // TEST!
      }
      
   }

   /*
    *  DrawablePolygon constructor #4
    *
    *  A regular n-gon with center (x,y) and radius r
    *
    */
    
   public DrawablePolygon( int x, int y, int r, int n ) {
       
      // invoke DrawablePolygon constructor #1:
      this();
      
      // rotation angle:
      double theta = 2 * Math.PI/n;
      
      // initial point is at twelve o'clock:
      DrawablePoint p = new DrawablePoint( 0, -r );
      this.addPoint( p );
      for ( int i = 1; i < n; i++ ) {
         // rotate the previous point:
         p = p.rotate( theta );
         this.addPoint( p );
      }
      this.translate( x, y );
      
   }

   // implement the methods of the Drawable interface:
   public Color getColor() { return color; }
   public void setColor( Color color ) {
      this.color = color;
   }
   public void draw( Graphics g ) {
      if ( this.color != null ) g.setColor( this.color );
      g.drawPolygon( this );
   }
   public void draw( Component c ) {
      this.draw( c.getGraphics() );
   }
   public void fill( Graphics g ) {
      if ( this.color != null ) g.setColor( this.color );
      g.fillPolygon( this );
   }
   public void fill( Component c ) {
      this.fill( c.getGraphics() );
   }
   
   /*
    *  Note:  All rotation methods mutate the current polygon!
    *
    */
    
   // rotate this polygon about the top lefthand corner
   // of its bounding box:
   public DrawablePolygon rotate( double theta ) {
   
      // return the rotated polygon as a side effect:
      return this.rotate( theta, new Point( this.getBounds().x,
                                            this.getBounds().y  ) );
      
   }
   
   // rotate this polygon about the center of its bounding box:
   public DrawablePolygon centerRotate( double theta ) {
   
      // compute the center of this polygon:
      java.awt.Rectangle r = this.getBounds();
      int x0 = r.x + r.width/2;
      int y0 = r.y + r.height/2;

      // return the rotated polygon as a side effect:
      return this.rotate( theta, new Point( x0, y0 ) );
      
   }
   
   // rotate this polygon about an arbitrary point:
   public DrawablePolygon rotate( double theta, Point p ) {  // should this by DrawablePoint?
                                                             // things will break!
      // local variables:
      int x, y; DrawablePoint q;

      // translate to the origin:
      this.translate( -p.x, -p.y );
      
      // rotate each vertex of this polygon:
      for ( int i = 0; i < this.npoints; i++ ) {
         x = this.xpoints[i]; y = this.ypoints[i];
         q = new DrawablePoint( x, y );
         q.rotate( theta );
         this.xpoints[i] = q.x;
         this.ypoints[i] = q.y;
      }
      
      // the bounding box is no longer valid:
      this.bounds = null;
      
      // translate back to original coordinates:
      this.translate( p.x, p.y );

      // return the rotated polygon as a side effect:
      return this;
      
   }

   // overload Polygon.addPoint( int, int ):
   public void addPoint( Point p ) {  // should this be DrawablePoint?
      super.addPoint( p.x, p.y );     // things will break!
   }
   
   // get all vertices of this polygon:
   public Point[] getPoints() {
   
      Point[] points = new Point[ npoints ];
      for ( int i = 0; i < npoints; i++ ) {
         points[i] = new Point( xpoints[i], ypoints[i] );
      }
      return points;
      
   }
   
}
