/*
 *  File:  DataServer.java
 * 
 *  This server is a prototype that shows the server allowing
 *  any number of clients to access a data resource.  These
 *  accesses are implemented as synchronized methods in the
 *  data class.
 *
 *  Message Protocol:  
 *  Server sends initial welcome response
 *  Each transaction consists of one request message from client
 *  followed by response message from server
 *  Request msg format:  
 *       "Vote: teamname" | "Results:"
 *  Response msg format:
 *       Response to vote:  "OK:" or "Error: message"
 *       Response to results:  "OK: list of teams and percents"
 *                             or "Error: message"
 *
 *  Usage:    % java DataServer [<port>]
 *
 */

import java.io.*;
import java.net.*;

class DataServer
{
   private static int port = 7775;  // default port

   public static void main( String[] args ) 
   {  
      checkArgs( args );

      // initialize data resource:  create instance of class
      VoteTeams voter = new VoteTeams();         
      try 
      {  
         // open a server socket:
         ServerSocket serversocket = new ServerSocket( port );

         while ( true )
         {  // wait for a connect message from client:
            Socket socket = serversocket.accept();
            // create a thread to open sockets to that client and
            // handle transaction
            // each client has a reference to the data resource
            ( new DataHandler( socket, voter ) ).start();
         }   
      }
      catch ( BindException be ) 
      {
         System.err.println( be );
         System.out.println( "Specify port on command line." );
      }
      catch ( IOException e ) 
      {
         System.err.println( e );
      } 

   } 

   private static void checkArgs( String[] args ) 
{
      // determine the port, if any:
      if ( args.length > 0 ) 
      { try 
        {
            port = Integer.parseInt( args[0] );
         } 
        catch ( NumberFormatException e ) 
        {
            System.err.println( e );
            System.out.println( "Ignoring bad command-line argument" );
         }
      }
   }
}  // end DataServer class

class DataHandler extends Thread
{
   // counts number of clients over all threads
   private static int numClients = 0;

   private Socket socket;
   private int id;
   private VoteTeams vote;
      
   DataHandler( Socket s, VoteTeams v ) 
   { socket = s; vote=v; id = ++numClients; }
   
   public void run()
   {  try 
      {  
         BufferedReader in =
           new BufferedReader(
             new InputStreamReader( socket.getInputStream() ) );
         boolean autoflush;
         PrintWriter out =
           new PrintWriter( socket.getOutputStream(), autoflush = true );

         out.println( "Hello" );

         //read requests from the client until connection closed
         while ( true )
         {  String str = in.readLine();
            if ( str == null ) break;
            if ( str.startsWith ("Vote:"))
              out.println ( vote.votefor(str.substring(5).trim()));
            else if (str.startsWith ("Results:"))
              out.println ( vote.results());
            else  out.println ( "Error: not a valid request");
         }
            
         // close the streams and socket:
         in.close();
         out.close();
         closeConnection();
      }
      catch ( IOException e ) 
      {
         System.err.println( e );
         closeConnection();
      } 
   } 

  private void closeConnection() 
  {
    try 
    { socket.close();
    } 
    catch ( IOException e ) 
   { System.err.println( e );
    }
  }
}  // end DataHandler class

class VoteTeams
{  // resource variables
  private String [] teams = {"Florida","Michigan St.","Wisconsin","North Carolina"};
  private int [] votes = {0,0,0,0};
  private int totalvotes = 0;

  public VoteTeams ()   { }

  public synchronized String votefor ( String teamname )
  {
    String result="Error: there is no team by that name";

    for (int i=0; i<teams.length; i++)
    { if (teams[i].equals(teamname))
	{ ++votes[i]; ++totalvotes; result = "OK:"; }
    }
  return result;
  }

  public synchronized String results()
  { String response = "";
    int percent;

    for (int i=0; i<teams.length; i++)
    { if (votes[i] != 0)
        { percent = (int)(votes[i]/(double)totalvotes*100);}
        else { percent = 0;}
      response += teams[i]+": "+percent+"% ";
    }
    return response;
  }
}  //end VoteTeams class
















